@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 // Create form.
 $expression_form = (new CForm())
 	->cleanItems()
@@ -29,7 +33,10 @@ $expression_form = (new CForm())
 	->addItem((new CVar('hostid', $data['hostid']))->removeId())
 	->addVar('groupid', $data['groupid'])
 	->addVar('itemid', $data['itemid'])
-	->addItem((new CInput('submit', 'submit'))->addStyle('display: none;'));
+	->addItem((new CInput('submit', 'submit'))
+		->addStyle('display: none;')
+		->removeId()
+	);
 
 if ($data['parent_discoveryid'] !== '') {
 	$expression_form->addVar('parent_discoveryid', $data['parent_discoveryid']);
@@ -45,26 +52,27 @@ $popup_options = [
 	'srcfld2' => 'name',
 	'dstfrm' => $expression_form->getName(),
 	'dstfld1' => 'itemid',
-	'dstfld2' => 'description',
+	'dstfld2' => 'item_description',
 	'with_webitems' => '1',
 	'writeonly' => '1'
 ];
-if ($data['groupid'] && $data['hostid']) {
-	$popup_options['groupid'] = $data['groupid'];
+
+if ($data['hostid']) {
 	$popup_options['hostid'] = $data['hostid'];
 }
+
 if ($data['parent_discoveryid'] !== '') {
 	$popup_options['normal_only'] = '1';
 }
 
 $item = [
-	(new CTextBox('description', $data['description'], true))
+	(new CTextBox('item_description', $data['item_description'], true))
 		->setAriaRequired()
 		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
 	(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 	(new CButton('select', _('Select')))
 		->addClass(ZBX_STYLE_BTN_GREY)
-		->onClick('return PopUp("popup.generic",'.CJs::encodeJson($popup_options).', null, this);')
+		->onClick('return PopUp("popup.generic",'.json_encode($popup_options).', null, this);')
 ];
 
 if ($data['parent_discoveryid'] !== '') {
@@ -72,20 +80,20 @@ if ($data['parent_discoveryid'] !== '') {
 	$item[] = (new CButton('select', _('Select prototype')))
 		->addClass(ZBX_STYLE_BTN_GREY)
 		->onClick('return PopUp("popup.generic",'.
-			CJs::encodeJson([
+			json_encode([
 				'srctbl' => 'item_prototypes',
 				'srcfld1' => 'itemid',
 				'srcfld2' => 'name',
 				'dstfrm' => $expression_form->getName(),
 				'dstfld1' => 'itemid',
-				'dstfld2' => 'description',
+				'dstfld2' => 'item_description',
 				'parent_discoveryid' => $data['parent_discoveryid']
 			]).', null, this);'
 		)
 		->removeId();
 }
 
-$expression_form_list->addRow((new CLabel(_('Item'), 'description'))->setAsteriskMark(), $item);
+$expression_form_list->addRow((new CLabel(_('Item'), 'item_description'))->setAsteriskMark(), $item);
 
 $function_combo_box = new CComboBox('function', $data['function'], 'reloadPopup(this.form, "popup.triggerexpr")');
 foreach ($data['functions'] as $id => $f) {
@@ -181,8 +189,7 @@ $output = [
 			'class' => '',
 			'keepOpen' => true,
 			'isSubmit' => true,
-			'action' => 'return validate_trigger_expression("expression", '.
-					'jQuery(window.document.forms["expression"]).closest("[data-dialogueid]").attr("data-dialogueid"));'
+			'action' => 'return validate_trigger_expression(overlay);'
 		]
 	],
 	'script_inline' =>
@@ -195,7 +202,7 @@ $output = [
 					'var tmp = elem.value;'.
 						'elem.value = value;'.
 
-					'"description" === elem.id && tmp !== value && reloadPopup(elem.form, "popup.triggerexpr");'.
+					'"item_description" === elem.id && tmp !== value && reloadPopup(elem.form, "popup.triggerexpr");'.
 				'}'.
 			'};'.
 		'});'
@@ -206,4 +213,4 @@ if ($data['user']['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
 	$output['debug'] = CProfiler::getInstance()->make()->toString();
 }
 
-echo (new CJson())->encode($output);
+echo json_encode($output);
