@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,8 +18,13 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 **/
 
+
+/**
+ * @var CView $this
+ */
+
 $pageHeader = (new CPageHeader(_('Fatal error, please report to the Zabbix team')))
-	->addCssFile('styles/'.CHtml::encode($data['theme']).'.css')
+	->addCssFile('assets/styles/'.CHtml::encode($data['theme']).'.css')
 	->display();
 
 $buttons = [
@@ -29,9 +34,11 @@ $buttons = [
 
 echo '<body lang="'.CWebUser::getLang().'">';
 
-(new CTag('main', true,
+(new CDiv((new CTag('main', true,
 	new CWarning(_('Fatal error, please report to the Zabbix team'), $data['messages'], $buttons)
-))->show();
+))))
+	->addClass(ZBX_STYLE_LAYOUT_WRAPPER)
+	->show();
 
 echo '</body>';
 echo '</html>';
