@@ -47,6 +47,8 @@ function getSystemStatusData(array $filter) {
 	$filter_ext_ack = array_key_exists('ext_ack', $filter)
 		? $filter['ext_ack']
 		: EXTACK_OPTION_ALL;
+	$filter_evaltype = array_key_exists('evaltype', $filter) ? $filter['evaltype'] : TAG_EVAL_TYPE_AND_OR;
+	$filter_tags = array_key_exists('tags', $filter) ? $filter['tags'] : [];
 
 	if (array_key_exists('exclude_groupids', $filter) && $filter['exclude_groupids']) {
 		if ($filter_hostids === null) {
@@ -113,6 +115,8 @@ function getSystemStatusData(array $filter) {
 		'output' => ['eventid', 'objectid', 'clock', 'ns', 'name', 'acknowledged', 'severity'],
 		'groupids' => array_keys($data['groups']),
 		'hostids' => $filter_hostids,
+		'evaltype' => $filter_evaltype,
+		'tags' => $filter_tags,
 		'source' => EVENT_SOURCE_TRIGGERS,
 		'object' => EVENT_OBJECT_TRIGGER,
 		'suppressed' => false,
@@ -323,11 +327,10 @@ function getSystemStatusData(array $filter) {
  * @param array  $data['triggers'][<triggerid>]['opdata']
  * @param array  $config
  * @param string $config['severity_name_*']
- * @param string $backurl
  *
  * @return CDiv
  */
-function makeSystemStatus(array $filter, array $data, array $config, $backurl) {
+function makeSystemStatus(array $filter, array $data, array $config) {
 	$filter_severities = (array_key_exists('severities', $filter) && $filter['severities'])
 		? $filter['severities']
 		: range(TRIGGER_SEVERITY_NOT_CLASSIFIED, TRIGGER_SEVERITY_COUNT - 1);
@@ -382,16 +385,16 @@ function makeSystemStatus(array $filter, array $data, array $config, $backurl) {
 			$allTriggersNum = $stat['count'];
 			if ($allTriggersNum) {
 				$allTriggersNum = (new CLinkAction($allTriggersNum))
-					->setHint(makeProblemsPopup($stat['problems'], $data['triggers'], $backurl, $data['actions'],
-						$config, $filter
+					->setHint(makeProblemsPopup($stat['problems'], $data['triggers'], $data['actions'], $config,
+						$filter
 					));
 			}
 
 			$unackTriggersNum = $stat['count_unack'];
 			if ($unackTriggersNum) {
 				$unackTriggersNum = (new CLinkAction($unackTriggersNum))
-					->setHint(makeProblemsPopup($stat['problems_unack'], $data['triggers'], $backurl, $data['actions'],
-						$config, $filter
+					->setHint(makeProblemsPopup($stat['problems_unack'], $data['triggers'], $data['actions'], $config,
+						$filter
 					));
 			}
 
@@ -542,7 +545,6 @@ function makeSeverityTotals(array $data) {
  * @param array   $data['filter']
  * @param array   $data['filter']['ext_ack']
  * @param array   $data['severity_names']
- * @param array   $data['backurl']
  * @param array   $stat
  * @param int     $stats['count']
  * @param array   $stats['problems']
@@ -563,16 +565,16 @@ function getSeverityTableCell($severity, array $data, array $stat, $is_total = f
 	$allTriggersNum = $stat['count'];
 	if ($allTriggersNum) {
 		$allTriggersNum = (new CLinkAction($allTriggersNum))
-			->setHint(makeProblemsPopup($stat['problems'], $data['data']['triggers'], $data['backurl'],
-				$data['data']['actions'], $data['severity_names'], $data['filter']
+			->setHint(makeProblemsPopup($stat['problems'], $data['data']['triggers'], $data['data']['actions'],
+				$data['severity_names'], $data['filter']
 			));
 	}
 
 	$unackTriggersNum = $stat['count_unack'];
 	if ($unackTriggersNum) {
 		$unackTriggersNum = (new CLinkAction($unackTriggersNum))
-			->setHint(makeProblemsPopup($stat['problems_unack'], $data['data']['triggers'], $data['backurl'],
-				$data['data']['actions'], $data['severity_names'], $data['filter']
+			->setHint(makeProblemsPopup($stat['problems_unack'], $data['data']['triggers'], $data['data']['actions'],
+				$data['severity_names'], $data['filter']
 			));
 	}
 
@@ -665,9 +667,13 @@ function make_status_of_zbx() {
 		]);
 	}
 
-	// check requirements
+	// Check requirements.
 	if (CWebUser::getType() == USER_TYPE_SUPER_ADMIN) {
-		foreach ((new CFrontendSetup())->checkRequirements() as $req) {
+		$setup = new CFrontendSetup();
+		$reqs = $setup->checkRequirements();
+		$reqs[] = $setup->checkSslFiles();
+
+		foreach ($reqs as $req) {
 			if ($req['result'] == CFrontendSetup::CHECK_FATAL) {
 				$table->addRow(
 					(new CRow([$req['name'], $req['current'], $req['error']]))->addClass(ZBX_STYLE_RED)
@@ -684,6 +690,17 @@ function make_status_of_zbx() {
 		}
 	}
 
+	// Warn if database history tables have not been upgraded.
+	global $DB;
+
+	if (!$DB['DOUBLE_IEEE754']) {
+		$table->addRow([
+			_('Database history tables upgraded'),
+			(new CSpan(_('No')))->addClass(ZBX_STYLE_RED),
+			''
+		]);
+	}
+
 	return $table;
 }
 
@@ -708,7 +725,6 @@ function make_status_of_zbx() {
  * @param array  $triggers[<triggerid>]['hosts']
  * @param string $triggers[<triggerid>]['hosts'][]['name']
  * @param string $triggers[<triggerid>]['opdata']
- * @param string $backurl
  * @param array  $actions
  * @param array  $config
  * @param array  $filter
@@ -718,7 +734,7 @@ function make_status_of_zbx() {
  *
  * @return CTableInfo
  */
-function makeProblemsPopup(array $problems, array $triggers, $backurl, array $actions, array $config, array $filter) {
+function makeProblemsPopup(array $problems, array $triggers, array $actions, array $config, array $filter) {
 	$url_details = (new CUrl('tr_events.php'))
 		->setArgument('triggerid', '')
 		->setArgument('eventid', '');
@@ -845,16 +861,12 @@ function makeProblemsPopup(array $problems, array $triggers, $backurl, array $ac
 			}
 		}
 
-		// ack
-		$problem_update_url = (new CUrl('zabbix.php'))
-			->setArgument('action', 'acknowledge.edit')
-			->setArgument('eventids', [$problem['eventid']])
-			->setArgument('backurl', $backurl)
-			->getUrl();
-
-		$ack = (new CLink($problem['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $problem_update_url))
-			->addClass($problem['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-			->addClass(ZBX_STYLE_LINK_ALT);
+		// Create acknowledge link.
+		$is_acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
+		$problem_update_link = (new CLink($is_acknowledged ? _('Yes') : _('No')))
+			->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+			->addClass(ZBX_STYLE_LINK_ALT)
+			->onClick('acknowledgePopUp('.json_encode(['eventids' => [$problem['eventid']]]).', this);');
 
 		$table->addRow(array_merge($row, [
 			makeInformationList($info_icons),
@@ -867,7 +879,7 @@ function makeProblemsPopup(array $problems, array $triggers, $backurl, array $ac
 			),
 			($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) ? $opdata : null,
 			zbx_date2age($problem['clock']),
-			$ack,
+			$problem_update_link,
 			makeEventActionsIcons($problem['eventid'], $actions['all_actions'], $actions['mediatypes'],
 				$actions['users'], $config
 			),
