@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -41,11 +41,17 @@ class CNewValidator {
 	 */
 	private $range_time_parser;
 
+	/**
+	 * A parser for a list of time periods separated by a semicolon.
+	 *
+	 * @var CTimePeriodsParser
+	 */
+	private $time_periods_parser;
+
 	public function __construct(array $input, array $rules) {
 		$this->input = $input;
 		$this->rules = $rules;
 		$this->validationRuleParser = new CValidationRule();
-		$this->range_time_parser = null;
 
 		$this->validate();
 	}
@@ -96,7 +102,7 @@ class CNewValidator {
 
 				case 'json':
 					if (array_key_exists($field, $this->input)) {
-						if (!is_string($this->input[$field]) || CJs::decodeJson($this->input[$field]) === null) {
+						if (!is_string($this->input[$field]) || json_decode($this->input[$field]) === null) {
 							$this->addError($fatal,
 								_s('Incorrect value for field "%1$s": %2$s.', $field, _('JSON string is expected'))
 							);
@@ -270,6 +276,32 @@ class CNewValidator {
 					}
 					break;
 
+				/*
+				 * 'time_periods' => true
+				 */
+				case 'time_periods':
+					if (array_key_exists($field, $this->input) && !$this->isTimePeriods($this->input[$field])) {
+						$this->addError($fatal,
+							_s('Incorrect value for field "%1$s": %2$s.', $field, _('a time period is expected'))
+						);
+						return false;
+					}
+					break;
+
+				/*
+				 * 'rgb' => true
+				 */
+				case 'rgb':
+					if (array_key_exists($field, $this->input) && !$this->isRgb($this->input[$field])) {
+						$this->addError($fatal,
+							_s('Incorrect value for field "%1$s": %2$s.', $field,
+								_('a hexadecimal colour code (6 symbols) is expected')
+							)
+						);
+						return false;
+					}
+					break;
+
 				/*
 				 * 'string' => true
 				 */
@@ -299,30 +331,27 @@ class CNewValidator {
 	}
 
 	private function is_id($value) {
-		if (1 != preg_match('/^[0-9]+$/', $value)) {
+		if (!preg_match('/^'.ZBX_PREG_INT.'$/', $value)) {
 			return false;
 		}
 
-		// between 0 and _I64_MAX
 		return (bccomp($value, '0') >= 0 && bccomp($value, ZBX_DB_MAX_ID) <= 0);
 	}
 
 	public static function is_int32($value) {
-		if (1 != preg_match('/^\-?[0-9]+$/', $value)) {
+		if (!preg_match('/^'.ZBX_PREG_INT.'$/', $value)) {
 			return false;
 		}
 
-		// between INT_MIN and INT_MAX
-		return (bccomp($value, ZBX_MIN_INT32) >= 0 && bccomp($value, ZBX_MAX_INT32) <= 0);
+		return ($value >= ZBX_MIN_INT32 && $value <= ZBX_MAX_INT32);
 	}
 
 	public static function is_uint64($value) {
-		if (1 != preg_match('/^[0-9]+$/', $value)) {
+		if (!preg_match('/^'.ZBX_PREG_INT.'$/', $value)) {
 			return false;
 		}
 
-		// between 0 and _UI64_MAX
-		return (bccomp($value, '0') >= 0 && bccomp($value, '18446744073709551615') <= 0);
+		return ($value >= 0 && bccomp($value, ZBX_MAX_UINT64) <= 0);
 	}
 
 	/**
@@ -351,6 +380,7 @@ class CNewValidator {
 
 				return (mb_strlen($value) <= $field_schema['length']);
 
+			case DB::FIELD_TYPE_NCLOB:
 			case DB::FIELD_TYPE_TEXT:
 				if ($flags & P_CRLF) {
 					$value = CRLFtoLF($value);
@@ -413,20 +443,12 @@ class CNewValidator {
 		return (is_string($value) && $this->check_db_value($table_schema['fields'][$field], $value, $flags));
 	}
 
-	private function isLeapYear($year) {
-		return (0 == $year % 4 && (0 != $year % 100 || 0 == $year % 400));
-	}
-
-	private function getDaysInMonth($year, $month) {
-		if (in_array($month, [4, 6, 9, 11], true)) {
-			return 30;
+	private function isTimePeriods($value) {
+		if ($this->time_periods_parser === null) {
+			$this->time_periods_parser = new CTimePeriodsParser(['usermacros' => true]);
 		}
 
-		if ($month == 2) {
-			return $this->isLeapYear($year) ? 29 : 28;
-		}
-
-		return 31;
+		return is_string($value) && $this->time_periods_parser->parse($value) == CParser::PARSE_SUCCESS;
 	}
 
 	private function isRangeTime($value) {
@@ -437,6 +459,10 @@ class CNewValidator {
 		return is_string($value) && $this->range_time_parser->parse($value) == CParser::PARSE_SUCCESS;
 	}
 
+	private function isRgb($value) {
+		return is_string($value) && preg_match('/^[A-F0-9]{6}$/', $value);
+	}
+
 	/**
 	 * Add validation error.
 	 *
