@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -46,15 +46,17 @@ class CWidgetHelper {
 	 */
 	public static function createFormList($dialogue_name, $type, $view_mode, $known_widget_types, $field_rf_rate) {
 		return (new CFormList())
-			->addRow((new CLabel(_('Type'), 'type')), [
-					(new CComboBox('type', $type, 'updateWidgetConfigDialogue()', $known_widget_types)),
+			->addItem((new CListItem([
+					(new CDiv(new CLabel(_('Type'), 'type')))->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
+					(new CDiv(new CComboBox('type', $type, 'updateWidgetConfigDialogue()', $known_widget_types)))
+						->addClass(ZBX_STYLE_TABLE_FORMS_TD_RIGHT),
 					(new CDiv((new CCheckBox('show_header'))
-						->setLabel(_('Show header'))
-						->setLabelPosition(CCheckBox::LABEL_POSITION_LEFT)
-						->setId('show_header')
-						->setChecked($view_mode == ZBX_WIDGET_VIEW_MODE_NORMAL)
-					))->addClass(ZBX_STYLE_TABLE_FORMS_SECOND_COLUMN)
-				]
+							->setLabel(_('Show header'))
+							->setLabelPosition(CCheckBox::LABEL_POSITION_LEFT)
+							->setId('show_header')
+							->setChecked($view_mode == ZBX_WIDGET_VIEW_MODE_NORMAL)))
+						->addClass(ZBX_STYLE_TABLE_FORMS_SECOND_COLUMN)
+				]))->addClass('table-forms-row-with-second-field')
 			)
 			->addRow(_('Name'),
 				(new CTextBox('name', $dialogue_name))
@@ -211,8 +213,7 @@ class CWidgetHelper {
 	 */
 	private static function getMultiselectField($field, $captions, $form_name, $object_name, $popup_options) {
 		$field_name = $field->getName().($field->isMultiple() ? '[]' : '');
-
-		return (new CMultiSelect([
+		$options = [
 			'name' => $field_name,
 			'object_name' => $object_name,
 			'multiple' => $field->isMultiple(),
@@ -224,7 +225,13 @@ class CWidgetHelper {
 				] + $popup_options
 			],
 			'add_post_js' => false
-		]))
+		];
+
+		if ($field instanceof CWidgetFieldMsHost && $field->filter_preselect_host_group_field) {
+			$options['popup']['filter_preselect_fields']['hostgroups'] = $field->filter_preselect_host_group_field;
+		}
+
+		return (new CMultiSelect($options))
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 			->setAriaRequired(self::isAriaRequired($field));
 	}
@@ -333,7 +340,7 @@ class CWidgetHelper {
 			(new CButton('select', _('Select')))
 				->addClass(ZBX_STYLE_BTN_GREY)
 				->onClick('return PopUp("popup.generic",'.
-					CJs::encodeJson($field->getPopupOptions($form_name)).', null, this);')
+					json_encode($field->getPopupOptions($form_name)).', null, this);')
 		];
 	}
 
@@ -395,34 +402,26 @@ class CWidgetHelper {
 
 	/**
 	 * @param CWidgetFieldSeverities $field
-	 * @param array $config
 	 *
-	 * @return CList
+	 * @return CSeverityCheckBoxList
 	 */
-	public static function getSeverities($field, $config) {
-		$severities = [];
-
-		for ($severity = TRIGGER_SEVERITY_NOT_CLASSIFIED; $severity < TRIGGER_SEVERITY_COUNT; $severity++) {
-			$severities[$severity] = getSeverityName($severity, $config);
-		}
-
-		return self::getCheckBoxList($field, $severities);
+	public static function getSeverities($field) {
+		return (new CSeverityCheckBoxList($field->getName()))
+			->setChecked($field->getValue())
+			->setEnabled(!($field->getFlags() & CWidgetField::FLAG_DISABLED))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
 	}
 
 	/**
 	 * @param CWidgetFieldCheckBoxList $field
-	 * @param array $config
+	 * @param array                    $list  Option list array.
 	 *
 	 * @return CList
 	 */
-	public static function getCheckBoxList($field, array $config) {
+	public static function getCheckBoxList($field, array $list) {
 		$checkbox_list = (new CList())->addClass(ZBX_STYLE_LIST_CHECK_RADIO);
 
-		if ($field->getOrientation() == CWidgetFieldCheckBoxList::ORIENTATION_HORIZONTAL) {
-			$checkbox_list->addClass(ZBX_STYLE_HOR_LIST);
-		}
-
-		foreach ($config as $key => $label) {
+		foreach ($list as $key => $label) {
 			$checkbox_list->addItem(
 				(new CCheckBox($field->getName().'[]', $key))
 					->setLabel($label)
@@ -539,6 +538,13 @@ class CWidgetHelper {
 	 * @return array
 	 */
 	public static function getApplicationSelector($field) {
+		$popup_options = json_encode($field->getFilterParameters());
+
+		if ($field->filter_preselect_host_field) {
+			$popup_options = 'jQuery.extend('.
+				$popup_options.', getFirstMultiselectValue("'.$field->filter_preselect_host_field.'"))';
+		}
+
 		return [
 			(new CTextBox($field->getName(), $field->getValue()))
 				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
@@ -547,9 +553,7 @@ class CWidgetHelper {
 			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 			(new CButton($field->getName().'_select', _('Select')))
 				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick(
-					'return PopUp("popup.generic", '.CJs::encodeJson($field->getFilterParameters()).', null, this);'
-				)
+				->onClick('return PopUp("popup.generic", '.$popup_options.', null, this);')
 		];
 	}
 
@@ -631,13 +635,15 @@ class CWidgetHelper {
 							->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 					))->addClass(ZBX_STYLE_COLUMN_50)
 				]))
-					->addClass(ZBX_STYLE_COLUMN_95)
-					->addClass(ZBX_STYLE_COLUMNS),
+					->addClass(ZBX_STYLE_COLUMNS)
+					->addClass(ZBX_STYLE_COLUMNS_NOWRAP)
+					->addClass(ZBX_STYLE_COLUMN_95),
 
 				(new CDiv(
 					(new CButton())
 						->setAttribute('title', _('Delete'))
 						->addClass(ZBX_STYLE_REMOVE_BTN)
+						->removeId()
 				))
 					->addClass(ZBX_STYLE_COLUMN_5)
 			]))
@@ -677,9 +683,7 @@ class CWidgetHelper {
 	 * @return CList
 	 */
 	public static function getGraphOverride($field, $form_name) {
-		$list = (new CList())
-			->addClass(ZBX_STYLE_OVERRIDES_LIST)
-			->setId('overrides');
+		$list = (new CList())->addClass(ZBX_STYLE_OVERRIDES_LIST);
 
 		$values = $field->getValue();
 
@@ -818,18 +822,17 @@ class CWidgetHelper {
 	 * Return javascript necessary to initialize CWidgetFieldGraphOverride field.
 	 *
 	 * @param CWidgetFieldGraphOverride $field
-	 * @param string                    $form_name  Form name in which override field is located.
 	 *
 	 * @return string
 	 */
-	public static function getGraphOverrideJavascript($field, $form_name) {
+	public static function getGraphOverrideJavascript($field) {
 		$scripts = [
 			// Define it as function to avoid redundancy.
 			'function initializeOverrides() {'.
 				'jQuery("#overrides .'.ZBX_STYLE_OVERRIDES_OPTIONS_LIST.'").overrides({'.
 					'add: ".'.ZBX_STYLE_BTN_ALT.'",'.
 					'options: "input[type=hidden]",'.
-					'captions: '.CJs::encodeJson(self::getGraphOverrideOptionNames()).','.
+					'captions: '.json_encode(self::getGraphOverrideOptionNames()).','.
 					'makeName: function(option, row_id) {'.
 						'return "'.$field->getName().'[" + row_id + "][" + option + "]";'.
 					'},'.
@@ -841,7 +844,7 @@ class CWidgetHelper {
 					'override: ".'.ZBX_STYLE_OVERRIDES_LIST_ITEM.'",'.
 					'overridesList: ".'.ZBX_STYLE_OVERRIDES_LIST.'",'.
 					'onUpdate: onGraphConfigChange,'.
-					'menu: '.CJs::encodeJson(self::getGraphOverrideMenu()).
+					'menu: '.json_encode(self::getGraphOverrideMenu()).
 				'});'.
 			'}',
 
@@ -856,7 +859,9 @@ class CWidgetHelper {
 				'})'.
 				'.bind("afteradd.dynamicRows", function(event, options) {'.
 					'var container = jQuery(".overlay-dialogue-body");'.
-					'container.scrollTop(container[0].scrollHeight);'.
+					'container.scrollTop(Math.max(container.scrollTop(),
+						jQuery("#widget_dialogue_form")[0].scrollHeight - container.height()
+					));'.
 
 					'jQuery(".multiselect", jQuery("#overrides")).each(function() {'.
 						'jQuery(this).multiSelect(jQuery(this).data("params"));'.
@@ -899,7 +904,7 @@ class CWidgetHelper {
 				'handle: ".drag-icon",'.
 				'tolerance: "pointer",'.
 				'scroll: false,'.
-				'cursor: IE ? "move" : "grabbing",'.
+				'cursor: "grabbing",'.
 				'opacity: 0.6,'.
 				'axis: "y",'.
 				'disabled: function() {'.
@@ -943,7 +948,8 @@ class CWidgetHelper {
 						(new CButton())
 							->addClass(ZBX_STYLE_COLOR_PREVIEW_BOX)
 							->addStyle('background-color: #'.$value['color'].';')
-							->setAttribute('title', $is_opened ? _('Collapse') : _('Expand')),
+							->setAttribute('title', $is_opened ? _('Collapse') : _('Expand'))
+							->removeId(),
 						(new CPatternSelect([
 							'name' => $field_name.'['.$row_num.'][hosts][]',
 							'object_name' => 'hosts',
@@ -982,12 +988,14 @@ class CWidgetHelper {
 						]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 					))->addClass(ZBX_STYLE_COLUMN_50),
 				]))
-					->addClass(ZBX_STYLE_COLUMN_95)
-					->addClass(ZBX_STYLE_COLUMNS),
+					->addClass(ZBX_STYLE_COLUMNS)
+					->addClass(ZBX_STYLE_COLUMNS_NOWRAP)
+					->addClass(ZBX_STYLE_COLUMN_95),
 				(new CDiv([
 					(new CButton())
 						->setAttribute('title', _('Delete'))
 						->addClass(ZBX_STYLE_REMOVE_BTN)
+						->removeId()
 				]))->addClass(ZBX_STYLE_COLUMN_5)
 			]))
 				->addClass(ZBX_STYLE_LIST_ACCORDION_ITEM_HEAD)
@@ -1118,6 +1126,7 @@ class CWidgetHelper {
 						->addClass(ZBX_STYLE_COLUMN_50),
 				]))
 					->addClass(ZBX_STYLE_COLUMNS)
+					->addClass(ZBX_STYLE_COLUMNS_NOWRAP)
 					->addClass(ZBX_STYLE_COLUMN_95)
 			))
 				->addClass(ZBX_STYLE_LIST_ACCORDION_ITEM_BODY)
@@ -1232,8 +1241,8 @@ class CWidgetHelper {
 					'row: ".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'",'.
 					'dataCallback: function(data) {'.
 						'data.color = function(num) {'.
-							'var palete = '.CWidgetFieldGraphDataSet::DEFAULT_COLOR_PALETE.';'.
-							'return palete[num % palete.length];'.
+							'var palette = '.CWidgetFieldGraphDataSet::DEFAULT_COLOR_PALETTE.';'.
+							'return palette[num % palette.length];'.
 						'} (data.rowNum);'.
 						'return data;'.
 					'}'.
@@ -1243,12 +1252,14 @@ class CWidgetHelper {
 				'})'.
 				'.bind("afteradd.dynamicRows", function(event, options) {'.
 					'var container = jQuery(".overlay-dialogue-body");'.
-					'container.scrollTop(container[0].scrollHeight);'.
+					'container.scrollTop(Math.max(container.scrollTop(),
+						jQuery("#widget_dialogue_form")[0].scrollHeight - container.height()
+					));'.
 
 					'jQuery(".input-color-picker input").colorpicker({onUpdate: function(color) {'.
 						'var ds = jQuery(this).closest(".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'");'.
 						'jQuery(".'.ZBX_STYLE_COLOR_PREVIEW_BOX.'", ds).css("background-color", "#"+color);'.
-					'}, appendTo: "#overlay_dialogue"});'.
+					'}, appendTo: ".overlay-dialogue-body"});'.
 
 					'jQuery(".multiselect", jQuery("#data_sets")).each(function() {'.
 						'jQuery(this).multiSelect(jQuery(this).data("params"));'.
@@ -1273,7 +1284,7 @@ class CWidgetHelper {
 					'}'.
 				'});',
 
-			// Intialize vertical accordion.
+			// Initialize vertical accordion.
 			'jQuery("#data_sets")'.
 				'.on("focus", ".'.CMultiSelect::ZBX_STYLE_CLASS.' input.input", function() {'.
 					'jQuery("#data_sets").zbx_vertical_accordion("expandNth",'.
@@ -1310,7 +1321,7 @@ class CWidgetHelper {
 			'jQuery(".input-color-picker input").colorpicker({onUpdate: function(color){'.
 				'var ds = jQuery(this).closest(".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'");'.
 				'jQuery(".'.ZBX_STYLE_COLOR_PREVIEW_BOX.'", ds).css("background-color", "#"+color);'.
-			'}, appendTo: "#overlay_dialogue"});',
+			'}, appendTo: ".overlay-dialogue-body"});',
 
 			// Initialize sortability.
 			'if (jQuery("#data_sets .'.ZBX_STYLE_LIST_ACCORDION_ITEM.'").length < 2) {'.
@@ -1322,7 +1333,7 @@ class CWidgetHelper {
 				'handle: ".drag-icon",'.
 				'tolerance: "pointer",'.
 				'scroll: false,'.
-				'cursor: IE ? "move" : "grabbing",'.
+				'cursor: "grabbing",'.
 				'opacity: 0.6,'.
 				'axis: "y",'.
 				'disabled: function() {'.
