@@ -781,7 +781,6 @@ function getItemFormData(array $item = [], array $options = []) {
 		'delay' => getRequest('delay', ZBX_ITEM_DELAY_DEFAULT),
 		'history' => getRequest('history', DB::getDefault('items', 'history')),
 		'status' => getRequest('status', isset($_REQUEST['form_refresh']) ? 1 : 0),
-		'discover' => getRequest('discover', DB::getDefault('items', 'discover')),
 		'type' => getRequest('type', 0),
 		'snmp_oid' => getRequest('snmp_oid', ''),
 		'value_type' => getRequest('value_type', ITEM_VALUE_TYPE_UINT64),
@@ -833,6 +832,15 @@ function getItemFormData(array $item = [], array $options = []) {
 		'preprocessing_script_maxlength' => DB::getFieldLength('item_preproc', 'params')
 	];
 
+	if ($data['parent_discoveryid'] != 0) {
+		$data['discover'] = hasRequest('form_refresh')
+			? getRequest('discover', DB::getDefault('items', 'discover'))
+			: ($item
+				? $item['discover']
+				: DB::getDefault('items', 'discover')
+			);
+	}
+
 	if ($data['type'] == ITEM_TYPE_HTTPAGENT) {
 		foreach (['query_fields', 'headers'] as $property) {
 			$values = [];
@@ -910,7 +918,6 @@ function getItemFormData(array $item = [], array $options = []) {
 		}
 		// item prototype
 		elseif ($data['parent_discoveryid'] != 0) {
-			$data['discover'] = $data['item']['discover'];
 			$flag = ZBX_FLAG_DISCOVERY_PROTOTYPE;
 		}
 		// plain item
