@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,10 +23,33 @@ function unset_request($key) {
 	unset($_GET[$key], $_POST[$key], $_REQUEST[$key]);
 }
 
-function BETWEEN($min, $max, $var = null) {
+/**
+ * Validation expression for min/max number range.
+ *
+ * @param int    $min
+ * @param int    $max
+ * @param string $var
+ *
+ * @return string
+ */
+function BETWEEN($min, $max, $var = '') {
 	return '({'.$var.'}>='.$min.'&&{'.$var.'}<='.$max.')&&';
 }
 
+/**
+ * Validation expression for min/max range and max number of digits after the decimal point.
+ *
+ * @param int    $min
+ * @param int    $max
+ * @param int    $scale
+ * @param string $var
+ *
+ * @return string
+ */
+function BETWEEN_DBL($min, $max, $scale, $var = '') {
+	return BETWEEN($min, $max, $var).'(round({'.$var.'},'.$scale.')=={'.$var.'})&&';
+}
+
 function IN($array, $var = '') {
 	if (is_array($array)) {
 		$array = implode(',', $array);
@@ -119,6 +142,8 @@ function unset_all() {
 }
 
 function check_type(&$field, $flags, &$var, $type, $caption = null) {
+	global $DB;
+
 	if ($caption === null) {
 		$caption = $field;
 	}
@@ -143,55 +168,32 @@ function check_type(&$field, $flags, &$var, $type, $caption = null) {
 		}
 	}
 	elseif ($type == T_ZBX_DBL) {
-		$decimalValidator = new CDecimalValidator([
-			'maxPrecision' => 16,
-			'maxScale' => 4,
-			'messageInvalid' => _('Value "%2$s" of "%1$s" has incorrect decimal format.'),
-			'messagePrecision' => _(
-				'Value "%2$s" of "%1$s" is too long: it cannot have more than %3$s digits before the decimal point '.
-				'and more than %4$s digits after the decimal point.'
-			),
-			'messageNatural' => _(
-				'Value "%2$s" of "%1$s" has too many digits before the decimal point: '.
-				'it cannot have more than %3$s digits.'
-			),
-			'messageScale' => _(
-				'Value "%2$s" of "%1$s" has too many digits after the decimal point: '.
-				'it cannot have more than %3$s digits.'
-			)
-		]);
-		$decimalValidator->setObjectName($caption);
+		$number_parser = new CNumberParser(['with_suffix' => false]);
 
-		if (!$decimalValidator->validate($var)) {
+		if ($number_parser->parse($var) != CParser::PARSE_SUCCESS) {
 			$error = true;
-			$message = $decimalValidator->getError();
+			$message = _s('Field "%1$s" is not correct: %2$s', $caption, _('a number is expected'));
 		}
-	}
-	elseif ($type == T_ZBX_DBL_BIG) {
-		$decimalValidator = new CDecimalValidator([
-			'maxScale' => 4,
-			'messageInvalid' => _('Value "%2$s" of "%1$s" has incorrect decimal format.'),
-			'messageScale' => _(
-				'Value "%2$s" of "%1$s" has too many digits after the decimal point: '.
-				'it cannot have more than %3$s digits.'
-			)
-		]);
-		$decimalValidator->setObjectName($caption);
 
-		if (!$decimalValidator->validate($var)) {
-			$error = true;
-			$message = $decimalValidator->getError();
+		$value = $number_parser->calcValue();
+
+		if ($DB['DOUBLE_IEEE754']) {
+			if (abs($value) > ZBX_FLOAT_MAX) {
+				$error = true;
+				$message = _s('Field "%1$s" is not correct: %2$s', $caption, _('a number is too large'));
+			}
 		}
-	}
-	elseif ($type == T_ZBX_DBL_STR) {
-		$decimalStringValidator = new CDecimalStringValidator([
-			'messageInvalid' => _('Value "%2$s" of "%1$s" has incorrect decimal format.')
-		]);
-		$decimalStringValidator->setObjectName($caption);
-
-		if (!$decimalStringValidator->validate($var)) {
-			$error = true;
-			$message = $decimalStringValidator->getError();
+		else {
+			if (abs($value) >= 1E+16) {
+				$error = true;
+				$message = _s('Field "%1$s" is not correct: %2$s', $caption, _('a number is too large'));
+			}
+			elseif ($value != round($value, 4)) {
+				$error = true;
+				$message = _s('Field "%1$s" is not correct: %2$s', $caption,
+					_('a number has too many fractional digits')
+				);
+			}
 		}
 	}
 	elseif ($type == T_ZBX_STR) {
@@ -200,22 +202,6 @@ function check_type(&$field, $flags, &$var, $type, $caption = null) {
 			$message = _s('Field "%1$s" is not string.', $caption);
 		}
 	}
-	elseif ($type == T_ZBX_CLR) {
-		$colorValidator = new CColorValidator();
-
-		if (!$colorValidator->validate($var)) {
-			$error = true;
-			$message = _s('Colour "%1$s" is not correct: expecting hexadecimal colour code (6 symbols).', $caption);
-		}
-	}
-	elseif ($type == T_ZBX_TP) {
-		$time_period_parser = new CTimePeriodsParser(['usermacros' => true]);
-
-		if ($time_period_parser->parse($var) != CParser::PARSE_SUCCESS) {
-			$error = true;
-			$message = _s('Field "%1$s" is not correct: %2$s', $caption, _('a time period is expected'));
-		}
-	}
 	elseif ($type == T_ZBX_TU) {
 		$simple_interval_parser = new CSimpleIntervalParser([
 			'usermacros' => ($flags & P_ALLOW_USER_MACRO),
@@ -343,13 +329,21 @@ function check_field(&$fields, &$field, $checks) {
 
 	if ((is_null($exception) || $except) && $validation && !calc_exp($fields, $field, $validation)) {
 		if ($validation == NOT_EMPTY) {
-			info(_s('Incorrect value for field "%1$s": cannot be empty.', $caption));
+			info(_s('Incorrect value for field "%1$s": %2$s.', $caption, _('cannot be empty')));
 		}
-
-		// check for BETWEEN() function pattern and extract numbers e.g. ({}>=0&&{}<=999)&&
-		elseif (preg_match('/\(\{\}\>=([0-9]*)\&\&\{\}\<=([0-9]*)\)\&\&/', $validation, $result)) {
-			info(_s('Incorrect value "%1$s" for "%2$s" field: must be between %3$s and %4$s.',
-				$_REQUEST[$field], $caption, $result[1], $result[2]));
+		// Check for BETWEEN() or BETWEEN_SCALE function pattern and extract min, max and scale numbers.
+		elseif (preg_match('/\(\{\}>=(?<min>\d+)&&\{\}<=(?<max>\d+)\)&&(\(round\(\{\},(?<scale>\d+)\)==\{\}\)&&)?/',
+				$validation, $matches)) {
+			if (array_key_exists('scale', $matches)) {
+				info(_s('Incorrect value "%1$s" for "%2$s" field: must be between %3$s and %4$s, and have no more than %5$s digits after the decimal point.',
+					$_REQUEST[$field], $caption, $matches['min'], $matches['max'], $matches['scale']
+				));
+			}
+			else {
+				info(_s('Incorrect value "%1$s" for "%2$s" field: must be between %3$s and %4$s.',
+					$_REQUEST[$field], $caption, $matches['min'], $matches['max']
+				));
+			}
 		}
 		elseif (is_scalar($_REQUEST[$field])) {
 			info(_s('Incorrect value "%1$s" for "%2$s" field.', $_REQUEST[$field], $caption));
@@ -378,7 +372,7 @@ function invalid_url($msg = null) {
 
 	require_once dirname(__FILE__).'/page_header.php';
 
-	// rollback reseted messages
+	// Rollback reset messages.
 	$ZBX_MESSAGES = $temp;
 
 	unset_all();
@@ -386,14 +380,20 @@ function invalid_url($msg = null) {
 	require_once dirname(__FILE__).'/page_footer.php';
 }
 
-function check_fields(&$fields, $show_messages = true, $add_messages_to_message_stack = false) {
+/**
+ * Validate request fields and return result flags.
+ *
+ * @param array $fields field schema together with validation rules
+ *
+ * @return integer appropriate result flags ZBX_VALID_OK | ZBX_VALID_ERROR | ZBX_VALID_WARNING
+ */
+function check_fields_raw(&$fields) {
 	// VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 	$system_fields = [
 		'sid' =>			[T_ZBX_STR, O_OPT, P_SYS, HEX(),		null],
 		'triggers_hash' =>	[T_ZBX_STR, O_OPT, P_SYS, NOT_EMPTY,	null],
 		'print' =>			[T_ZBX_INT, O_OPT, P_SYS, IN('1'),		null],
-		'page' =>			[T_ZBX_INT, O_OPT, P_SYS, null,		null], // paging
-		'ddreset' =>		[T_ZBX_INT, O_OPT, P_SYS, null,		null]
+		'page' =>			[T_ZBX_INT, O_OPT, P_SYS, null,		null]	// paging
 	];
 	$fields = zbx_array_merge($system_fields, $fields);
 
@@ -411,15 +411,26 @@ function check_fields(&$fields, $show_messages = true, $add_messages_to_message_
 
 	$fields = null;
 
-	if ($err&ZBX_VALID_ERROR) {
+	return $err;
+}
+
+/**
+ * Validate request fields and return true on success, false on error.
+ *
+ * @param array $fields field schema together with validation rules
+ * @param bool $show_messages do show messages on error
+ *
+ * @return bool true on success, false on error.
+ */
+function check_fields(&$fields, $show_messages = true) {
+	$err = check_fields_raw($fields);
+
+	if ($err & ZBX_VALID_ERROR) {
 		invalid_url();
 	}
 
 	if ($show_messages && $err != ZBX_VALID_OK) {
-		show_messages(($err == ZBX_VALID_OK), null, _('Page received incorrect data'));
-	}
-	elseif ($add_messages_to_message_stack && $err != ZBX_VALID_OK) {
-		error(_('Page received incorrect data'));
+		show_messages(false, null, _('Page received incorrect data'));
 	}
 
 	return ($err == ZBX_VALID_OK);
@@ -448,14 +459,14 @@ function validateTimeSelectorPeriod($from, $to) {
 
 	if ($period < ZBX_MIN_PERIOD) {
 		error(_n('Minimum time period to display is %1$s minute.',
-			'Minimum time period to display is %1$s minutes.', (int) ZBX_MIN_PERIOD / SEC_PER_MIN
+			'Minimum time period to display is %1$s minutes.', (int) (ZBX_MIN_PERIOD / SEC_PER_MIN)
 		));
 
 		invalid_url();
 	}
 	elseif ($period > ZBX_MAX_PERIOD) {
 		error(_n('Maximum time period to display is %1$s day.',
-			'Maximum time period to display is %1$s days.', (int) ZBX_MAX_PERIOD / SEC_PER_DAY
+			'Maximum time period to display is %1$s days.', (int) (ZBX_MAX_PERIOD / SEC_PER_DAY)
 		));
 
 		invalid_url();
@@ -540,7 +551,7 @@ function validateDateInterval($year, $month, $day) {
  * If the value is incorrect, set an error.
  *
  * @param string $value                  Value to parse and validate.
- * @param int    $min                    Lowed bound.
+ * @param int    $min                    Lower bound.
  * @param int    $max                    Upper bound.
  * @param bool   $allow_zero             Set to "true" to allow value to be zero.
  * @param string $error
@@ -552,6 +563,7 @@ function validateDateInterval($year, $month, $day) {
  */
 function validateTimeUnit($value, $min, $max, $allow_zero, &$error, array $options = []) {
 	$simple_interval_parser = new CSimpleIntervalParser($options);
+	$value = (string) $value;
 
 	if ($simple_interval_parser->parse($value) == CParser::PARSE_SUCCESS) {
 		if ($value[0] !== '{') {
