@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -73,68 +73,64 @@ function getMenuPopupHistory(options) {
 /**
  * Get menu popup host section data.
  *
- * @param string options['hostid']           Host ID.
- * @param array  options['scripts']          Host scripts (optional).
- * @param string options[]['name']           Script name.
- * @param string options[]['scriptid']       Script ID.
- * @param string options[]['confirmation']   Confirmation text.
- * @param bool   options['showGraphs']       Link to host graphs page.
- * @param bool   options['showScreens']      Link to host screen page.
- * @param bool   options['showTriggers']     Link to Monitoring->Problems page.
- * @param bool   options['hasGoTo']          "Go to" block in popup.
- * @param int    options['severity_min']     (optional)
- * @param bool   options['show_suppressed']  (optional)
- * @param array  options['urls']             (optional)
- * @param string options['url'][]['label']
- * @param string options['url'][]['url']
- * @param {object} trigger_elmnt             UI element which triggered opening of overlay dialogue.
+ * @param {string} options['hostid']             Host ID.
+ * @param {array}  options['scripts']            Host scripts (optional).
+ * @param {string} options[]['name']             Script name.
+ * @param {string} options[]['scriptid']         Script ID.
+ * @param {string} options[]['confirmation']     Confirmation text.
+ * @param {bool}   options['showGraphs']         Link to Monitoring->Hosts->Graphs page.
+ * @param {bool}   options['showScreens']        Link to Monitoring->Screen page.
+ * @param {bool}   options['showWeb']		     Link to Monitoring->Hosts->Web page.
+ * @param {bool}   options['showTriggers']       Link to Monitoring->Problems page.
+ * @param {bool}   options['hasGoTo']            "Go to" block in popup.
+ * @param {array}  options['severities']         (optional)
+ * @param {bool}   options['show_suppressed']    (optional)
+ * @param {array}  options['urls']               (optional)
+ * @param {string} options['url'][]['label']
+ * @param {string} options['url'][]['url']
+ * @param {string} options['filter_application'] (optional) Application name for filter by application.
+ * @param {object} trigger_elmnt                 UI element which triggered opening of overlay dialogue.
  *
  * @return array
  */
 function getMenuPopupHost(options, trigger_elmnt) {
 	var sections = [];
 
-	// scripts
-	if (typeof options.scripts !== 'undefined') {
-		sections.push({
-			label: t('Scripts'),
-			items: getMenuPopupScriptData(options.scripts, options.hostid, trigger_elmnt)
-		});
-	}
-
 	// go to section
 	if (options.hasGoTo) {
-		// inventory
 		var	host_inventory = {
-				label: t('Host inventory')
+				label: t('Inventory')
 			},
-			host_inventory_url = new Curl('hostinventories.php', false),
-			// latest
 			latest_data = {
 				label: t('Latest data')
 			},
-			latest_data_url = new Curl('latest.php', false),
-			// problems
 			problems = {
 				label: t('Problems')
 			},
-			// graphs
 			graphs = {
 				label: t('Graphs')
 			},
-			// screens
 			screens = {
-				label: t('Host screens')
+				label: t('Screens')
+			},
+			web = {
+				label: t('Web')
 			};
 
 		// inventory link
-		host_inventory_url.setArgument('hostid', options.hostid);
-		host_inventory.url = host_inventory_url.getUrl();
+		var url = new Curl('hostinventories.php', false);
+		url.setArgument('hostid', options.hostid);
+		host_inventory.url = url.getUrl();
 
 		// latest data link
-		latest_data_url.setArgument('hostids[]', options.hostid);
-		latest_data_url.setArgument('filter_set', '1');
-		latest_data.url = latest_data_url.getUrl();
+		var url = new Curl('zabbix.php', false);
+		url.setArgument('action', 'latest.view');
+		if (typeof options.filter_application !== 'undefined') {
+			url.setArgument('filter_application', options.filter_application);
+		}
+		url.setArgument('filter_hostids[]', options.hostid);
+		url.setArgument('filter_set', '1');
+		latest_data.url = url.getUrl();
 
 		if (!options.showTriggers) {
 			problems.disabled = true;
@@ -143,12 +139,15 @@ function getMenuPopupHost(options, trigger_elmnt) {
 			var url = new Curl('zabbix.php', false);
 			url.setArgument('action', 'problem.view');
 			url.setArgument('filter_hostids[]', options.hostid);
-			if (typeof options.severity_min !== 'undefined') {
-				url.setArgument('filter_severity', options.severity_min);
+			if (typeof options.severities !== 'undefined') {
+				url.setArgument('filter_severities[]', options.severities);
 			}
 			if (typeof options.show_suppressed !== 'undefined' && options.show_suppressed) {
 				url.setArgument('filter_show_suppressed', '1');
 			}
+			if (typeof options.filter_application !== 'undefined') {
+				url.setArgument('filter_application', options.filter_application);
+			}
 			url.setArgument('filter_set', '1');
 			problems.url = url.getUrl();
 		}
@@ -157,9 +156,13 @@ function getMenuPopupHost(options, trigger_elmnt) {
 			graphs.disabled = true;
 		}
 		else {
-			var graphs_url = new Curl('charts.php', false);
+			var graphs_url = new Curl('zabbix.php', false);
 
-			graphs_url.setArgument('hostid', options.hostid);
+			graphs_url.setArgument('action', 'charts.view')
+			graphs_url.setArgument('view_as', 'showgraph'); // HISTORY_GRAPH
+			graphs_url.setArgument('filter_search_type', '0'); // ZBX_SEARCH_TYPE_STRICT
+			graphs_url.setArgument('filter_hostids[]', options.hostid);
+			graphs_url.setArgument('filter_set', '1');
 			graphs.url = graphs_url.getUrl();
 		}
 
@@ -173,15 +176,47 @@ function getMenuPopupHost(options, trigger_elmnt) {
 			screens.url = screens_url.getUrl();
 		}
 
+		if (!options.showWeb) {
+			web.disabled = true;
+		}
+		else {
+			var web_url = new Curl('zabbix.php', false);
+			web_url.setArgument('action', 'web.view');
+			web_url.setArgument('hostid', options.hostid);
+
+			web.url = web_url.getUrl();
+		}
+
+		var items = [
+			host_inventory,
+			latest_data,
+			problems,
+			graphs,
+			screens,
+			web
+		];
+
+		if (options.showConfig) {
+			var config = {
+				label: t('Configuration')
+			};
+
+			if (options.isWriteable) {
+				var config_url = new Curl('hosts.php', false);
+				config_url.setArgument('form', 'update');
+				config_url.setArgument('hostid', options.hostid);
+				config.url = config_url.getUrl();
+			}
+			else {
+				config.disabled = true;
+			}
+
+			items.push(config);
+		}
+
 		sections.push({
-			label: t('Go to'),
-			items: [
-				host_inventory,
-				latest_data,
-				problems,
-				graphs,
-				screens
-			]
+			label: t('Host'),
+			items: items
 		});
 	}
 
@@ -193,6 +228,14 @@ function getMenuPopupHost(options, trigger_elmnt) {
 		});
 	}
 
+	// scripts
+	if (typeof options.scripts !== 'undefined') {
+		sections.push({
+			label: t('Scripts'),
+			items: getMenuPopupScriptData(options.scripts, options.hostid, trigger_elmnt)
+		});
+	}
+
 	return sections;
 }
 
@@ -201,6 +244,7 @@ function getMenuPopupHost(options, trigger_elmnt) {
  *
  * @param {array}  options['sysmapid']
  * @param {int}    options['severity_min']     (optional)
+ * @param {int}    options['widget_uniqueid']  (optional)
  * @param {array}  options['urls']             (optional)
  * @param {string} options['url'][]['label']
  * @param {string} options['url'][]['url']
@@ -209,12 +253,19 @@ function getMenuPopupHost(options, trigger_elmnt) {
  */
 function getMenuPopupMapElementSubmap(options) {
 	var sections = [],
-		submap_url = new Curl('zabbix.php', false);
+		submap_url;
 
-	submap_url.setArgument('action', 'map.view');
-	submap_url.setArgument('sysmapid', options.sysmapid);
-	if (typeof options.severity_min !== 'undefined') {
-		submap_url.setArgument('severity_min', options.severity_min);
+	if (typeof options.widget_uniqueid !== 'undefined') {
+		submap_url = new Curl('javascript: navigateToSubmap(' + options.sysmapid +
+			', "' + options.widget_uniqueid + '");', false);
+	}
+	else {
+		submap_url = new Curl('zabbix.php', false);
+		submap_url.setArgument('action', 'map.view');
+		submap_url.setArgument('sysmapid', options.sysmapid);
+		if (typeof options.severity_min !== 'undefined') {
+			submap_url.setArgument('severity_min', options.severity_min);
+		}
 	}
 
 	sections.push({
@@ -240,11 +291,12 @@ function getMenuPopupMapElementSubmap(options) {
  * Get menu popup host group map element section data.
  *
  * @param {string} options['groupid']
- * @param {int}    options['severity_min']     (optional)
- * @param {bool}   options['show_suppressed']  (optional)
- * @param {array}  options['urls']             (optional)
+ * @param {array}  options['severities']         (optional)
+ * @param {bool}   options['show_suppressed']    (optional)
+ * @param {array}  options['urls']               (optional)
  * @param {string} options['url'][]['label']
  * @param {string} options['url'][]['url']
+ * @param {string} options['filter_application'] (optional) Application name for filter by application.
  *
  * @return array
  */
@@ -254,12 +306,15 @@ function getMenuPopupMapElementGroup(options) {
 
 	problems_url.setArgument('action', 'problem.view');
 	problems_url.setArgument('filter_groupids[]', options.groupid);
-	if (typeof options.severity_min !== 'undefined') {
-		problems_url.setArgument('severity_min', options.severity_min);
+	if (typeof options.severities !== 'undefined') {
+		problems_url.setArgument('filter_severities[]', options.severities);
 	}
 	if (typeof options.show_suppressed !== 'undefined' && options.show_suppressed) {
 		problems_url.setArgument('filter_show_suppressed', '1');
 	}
+	if (typeof options.filter_application !== 'undefined') {
+		problems_url.setArgument('filter_application', options.filter_application);
+	}
 	problems_url.setArgument('filter_set', '1');
 
 	sections.push({
@@ -285,7 +340,7 @@ function getMenuPopupMapElementGroup(options) {
  * Get menu popup trigger map element section data.
  *
  * @param {array}  options['triggerids']
- * @param {int}    options['severity_min']     (optional)
+ * @param {array}  options['severities']       (optional)
  * @param {bool}   options['show_suppressed']  (optional)
  * @param {array}  options['urls']             (optional)
  * @param {string} options['url'][]['label']
@@ -299,8 +354,8 @@ function getMenuPopupMapElementTrigger(options) {
 
 	problems_url.setArgument('action', 'problem.view');
 	problems_url.setArgument('filter_triggerids[]', options.triggerids);
-	if (typeof options.severity_min !== 'undefined') {
-		problems_url.setArgument('filter_severity', options.severity_min);
+	if (typeof options.severities !== 'undefined') {
+		problems_url.setArgument('filter_severities[]', options.severities);
 	}
 	if (typeof options.show_suppressed !== 'undefined' && options.show_suppressed) {
 		problems_url.setArgument('filter_show_suppressed', '1');
@@ -390,8 +445,8 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 				value: value
 			},
 			clickCallback: function() {
-				var obj = jQuery(this),
-					currentRate = obj.data('value');
+				var $obj = jQuery(this),
+					currentRate = $obj.data('value');
 
 				// it is a quick solution for slide refresh multiplier, should be replaced with slide.refresh or similar
 				if (options.multiplier) {
@@ -401,10 +456,13 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 							widgetRefreshRate: currentRate
 						}),
 						dataType: 'script',
-						success: function(js) { js }
+						success: function() {
+							// Set new refresh rate as current in slideshow controls.
+							trigger_elmnt.data('menu-popup').data.currentRate = currentRate;
+						}
 					});
 
-					jQuery('a', obj.closest('.action-menu')).each(function() {
+					jQuery('a', $obj.closest('.menu-popup')).each(function() {
 						var link = jQuery(this);
 
 						if (link.data('value') == currentRate) {
@@ -419,7 +477,7 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 						}
 					});
 
-					obj.closest('.action-menu').menuPopup('close', trigger_elmnt);
+					$obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
 				}
 				else {
 					var url = new Curl('zabbix.php');
@@ -433,8 +491,8 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 							'widgetid': options.widgetName,
 							'rf_rate': currentRate
 						},
-						success: function(resp) {
-							jQuery('a', obj.closest('.action-menu')).each(function() {
+						success: function() {
+							jQuery('a', $obj.closest('.menu-popup')).each(function() {
 								var link = jQuery(this);
 
 								if (link.data('value') == currentRate) {
@@ -449,13 +507,16 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 								}
 							});
 
-							obj.closest('.action-menu').menuPopup('close', trigger_elmnt);
+							// Set new refresh rate as current in widget controls.
+							trigger_elmnt.data('menu-popup').data.currentRate = currentRate;
+
+							$obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
 
 							jQuery('.dashbrd-grid-container')
 								.dashboardGrid('setWidgetRefreshRate', options.widgetName, parseInt(currentRate));
 						},
 						error: function() {
-							obj.closest('.action-menu').menuPopup('close', trigger_elmnt);
+							$obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
 							// TODO: gentle message about failed saving of widget refresh rate
 						}
 					});
@@ -476,6 +537,88 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 	}];
 }
 
+/**
+ * Get menu popup widget actions data.
+ *
+ * @param {string}   options['currentRate']      Current rate value.
+ * @param {bool}     options['multiplier']       Multiplier or time mode.
+ * @param {callback} options['callback']         Callback function on success (optional).
+ * @param {string}   options['widget_uniqueid']  Widget instance unique id.
+ * @param {object}   trigger_elmnt               UI element which triggered opening of overlay dialogue.
+ *
+ * @return array
+ */
+function getMenuPopupWidgetActions(options, trigger_elmnt) {
+	var $dashboard = jQuery('.dashbrd-grid-container'),
+		editMode = $dashboard.dashboardGrid('isEditMode'),
+		widget = $dashboard.dashboardGrid('getWidgetsBy', 'uniqueid', options.widget_uniqueid).pop(),
+		widgetid = widget.widgetid,
+		loading = (!widget['ready'] || widget['content_body'].find('.is-loading').length > 0),
+		menu = editMode ? [] : getMenuPopupRefresh(options, trigger_elmnt),
+		widget_actions = [];
+
+	widget_actions.push({
+		label: t('S_COPY'),
+		clickCallback: function() {
+			jQuery('.dashbrd-grid-container').dashboardGrid('copyWidget', widget);
+			jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
+			jQuery('#dashbrd-paste-widget').attr('disabled', false);
+		}
+	});
+
+	if (editMode) {
+		widget_actions.push({
+			label: t('S_PASTE'),
+			disabled: !$dashboard.dashboardGrid('isWidgetCopied'),
+			clickCallback: function() {
+				$dashboard.dashboardGrid('pasteWidget', widget, widget.pos);
+				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
+			}
+		});
+
+		widget_actions.push({
+			label: t('Delete'),
+			clickCallback: function() {
+				jQuery('.dashbrd-grid-container').dashboardGrid('deleteWidget', widget);
+				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
+			}
+		});
+	}
+
+	if ('download' in options && !editMode) {
+		widget_actions.push({
+			label: t('Download image'),
+			disabled: loading || !options.download,
+			clickCallback: function() {
+				var svg = widget['content_body'].find('svg').first();
+
+				if (svg.length) {
+					downloadSvgImage(svg, 'graph.png');
+				}
+				else {
+					downloadPngImage(widget['content_body'].find('img').first(), 'graph.png');
+				}
+
+				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
+			},
+			refreshCallback: function(widget) {
+				if (widget['widgetid'] == widgetid && options.download) {
+					this.disabled = !widget['ready'];
+				}
+			}
+		});
+	}
+
+	if (widget_actions.length) {
+		menu.unshift({
+			label: t('Actions'),
+			items: widget_actions
+		});
+	}
+
+	return menu;
+}
+
 /**
  * Get menu popup trigger section data.
  *
@@ -508,7 +651,7 @@ function getMenuPopupDashboard(options, trigger_elmnt) {
 					var popup_options = {'dashboardid': options.dashboardid};
 					PopUp('dashboard.share.edit', popup_options, 'dashboard_share', trigger_elmnt);
 
-					jQuery(this).closest('.action-menu').menuPopup('close', null);
+					jQuery(this).closest('.menu-popup').menuPopup('close', null);
 				},
 				disabled: !options.editable
 			},
@@ -527,7 +670,7 @@ function getMenuPopupDashboard(options, trigger_elmnt) {
 					var	obj = jQuery(this);
 
 					// hide menu
-					obj.closest('.action-menu').hide();
+					obj.closest('.menu-popup').hide();
 
 					if (!confirm(t('Delete dashboard?'))) {
 						return false;
@@ -545,16 +688,13 @@ function getMenuPopupDashboard(options, trigger_elmnt) {
  * Get menu popup trigger section data.
  *
  * @param {string} options['triggerid']               Trigger ID.
- * @param {string} options['eventid']                 (optional) Required for Acknowledge and Description sections.
+ * @param {string} options['eventid']                 (optional) Required for Acknowledge section.
  * @param {object} options['items']                   Link to trigger item history page (optional).
  * @param {string} options['items'][]['name']         Item name.
  * @param {object} options['items'][]['params']       Item URL parameters ("name" => "value").
- * @param {object} options['acknowledge']             Link to acknowledge page (optional).
- * @param {string} options['acknowledge']['backurl']  Return URL.
+ * @param {bool}   options['acknowledge']             (optional) Whether to show Acknowledge section.
  * @param {object} options['configuration']           Link to trigger configuration page (optional).
  * @param {bool}   options['showEvents']              Show Problems item enabled. Default: false.
- * @param {bool}   options['show_description']        Show Description item in context menu. Default: true.
- * @param {bool}   options['description_enabled']     Show Description item enabled. Default: true.
  * @param {string} options['url']                     Trigger URL link (optional).
  * @param {object} trigger_elmnt                      UI element which triggered opening of overlay dialogue.
  *
@@ -584,42 +724,15 @@ function getMenuPopupTrigger(options, trigger_elmnt) {
 	items[items.length] = events;
 
 	// acknowledge
-	if (typeof options.acknowledge !== 'undefined' && objectSize(options.acknowledge) > 0) {
-		var url = new Curl('zabbix.php', false);
-
-		url.setArgument('action', 'acknowledge.edit');
-		url.setArgument('eventids[]', options.eventid);
-		url.setArgument('backurl', options.acknowledge.backurl);
-
+	if (typeof options.acknowledge !== 'undefined' && options.acknowledge) {
 		items[items.length] = {
 			label: t('Acknowledge'),
-			url: url.getUrl()
-		};
-	}
+			clickCallback: function() {
+				jQuery(this).closest('.menu-popup-top').menuPopup('close', null);
 
-	// description
-	if (typeof options.show_description === 'undefined' || options.show_description !== false) {
-		var trigger_descr = {
-			label: t('Description')
-		};
-
-		if (typeof options.description_enabled === 'undefined' || options.description_enabled !== false) {
-			trigger_descr.clickCallback = function() {
-				var	popup_options = {triggerid: options.triggerid};
-
-				if (typeof options.eventid !== 'undefined') {
-					popup_options.eventid = options.eventid;
-				}
-
-				jQuery(this).closest('.action-menu').menuPopup('close', null);
-
-				return PopUp('popup.trigdesc.view', popup_options, null, trigger_elmnt);
+				acknowledgePopUp({eventids: [options.eventid]}, trigger_elmnt);
 			}
-		}
-		else {
-			trigger_descr.disabled = true;
-		}
-		items[items.length] = trigger_descr;
+		};
 	}
 
 	// configuration
@@ -635,19 +748,19 @@ function getMenuPopupTrigger(options, trigger_elmnt) {
 		};
 	}
 
-	// url
-	if (typeof options.url !== 'undefined' && options.url.length > 0) {
-		items[items.length] = {
-			label: t('URL'),
-			url: options.url
-		};
-	}
-
 	sections[sections.length] = {
 		label: t('S_TRIGGER'),
 		items: items
 	};
 
+	// urls
+	if ('urls' in options) {
+		sections[sections.length] = {
+			label: t('Links'),
+			items: options.urls
+		};
+	}
+
 	// items
 	if (typeof options.items !== 'undefined' && objectSize(options.items) > 0) {
 		var items = [];
@@ -694,7 +807,7 @@ function getMenuPopupItem(options, trigger_elmnt) {
 		items.push({
 			label: t('Create trigger'),
 			clickCallback: function() {
-				jQuery(this).closest('.action-menu').menuPopup('close', null);
+				jQuery(this).closest('.menu-popup').menuPopup('close', null);
 
 				return PopUp('popup.triggerwizard', {
 					itemid: options.itemid
@@ -714,7 +827,7 @@ function getMenuPopupItem(options, trigger_elmnt) {
 				triggers.push({
 					label: trigger.name,
 					clickCallback: function() {
-						jQuery(this).closest('.action-menu-top').menuPopup('close', null);
+						jQuery(this).closest('.menu-popup-top').menuPopup('close', null);
 
 						return PopUp('popup.triggerwizard', {
 							itemid: options.itemid,
@@ -790,6 +903,85 @@ function getMenuPopupItemPrototype(options) {
 	}];
 }
 
+/**
+ * Get dropdown section data.
+ *
+ * @param {array}  options
+ * @param {object} trigger_elem  UI element that was clicked to open overlay dialogue.
+ *
+ * @returns array
+ */
+function getMenuPopupDropdown(options, trigger_elem) {
+	var items = [];
+
+	jQuery.each(options.items, function(i, item) {
+		items.push({
+			label: item.label,
+			url: item.url || 'javascript:void(0);',
+			class: item.class,
+			clickCallback: () => {
+				jQuery(trigger_elem)
+					.removeClass()
+					.addClass(['btn-alt', 'btn-dropdown-toggle', item.class].join(' '));
+
+				jQuery('input[type=hidden]', jQuery(trigger_elem).parent())
+					.val(item.value)
+					.trigger('change');
+			}
+		});
+	});
+
+	return [{
+		items: items
+	}];
+}
+
+/**
+ * Get menu popup submenu section data.
+ *
+ * @param object options['submenu']                                    List of menu sections.
+ * @param object options['submenu'][section]                           An individual section definition.
+ * @param string options['submenu'][section]['label']                  Non-clickable section label.
+ * @param object options['submenu'][section]['items']                  List of menu items of the section.
+ * @param string options['submenu'][section]['items'][url]             Menu item label for the given url.
+ * @param object options['submenu'][section]['items'][key]             Menu item with a submenu.
+ * @param object options['submenu'][section]['items'][key]['label']    Non-clickable subsection label.
+ * @param object options['submenu'][section]['items'][key]['items']    List of menu items of the subsection.
+ * @param object options['submenu'][section]['items'][key]['items'][]  More levels of submenu.
+ *
+ * @returns array
+ */
+function getMenuPopupSubmenu(options) {
+	var transform = function(sections) {
+			var result = [];
+
+			for (var key in sections) {
+				if (typeof sections[key] === 'object') {
+					var item = {};
+					for (var item_key in sections[key]) {
+						if (item_key === 'items') {
+							item[item_key] = transform(sections[key][item_key]);
+						}
+						else {
+							item[item_key] = sections[key][item_key];
+						}
+					}
+					result.push(item);
+				}
+				else {
+					result.push({
+						'label': sections[key],
+						'url': key
+					});
+				}
+			}
+
+			return result;
+		};
+
+	return transform(options.submenu);
+}
+
 /**
  * Get data for the "Insert expression" menu in the trigger expression constructor.
  *
@@ -820,7 +1012,7 @@ function getMenuPopupTriggerMacro(options) {
 
 				expressionInput.val(expression.string);
 
-				jQuery(this).closest('.action-menu').menuPopup('close', null);
+				jQuery(this).closest('.menu-popup').menuPopup('close', null);
 			}
 		};
 	});
@@ -891,7 +1083,7 @@ function getMenuPopupScriptData(scripts, hostId, trigger_elmnt) {
 
 				if (typeof data.params !== 'undefined' && typeof data.params.scriptId !== 'undefined') {
 					item.clickCallback = function(e) {
-						jQuery(this).closest('.action-menu-top').menuPopup('close', trigger_elm, false);
+						jQuery(this).closest('.menu-popup-top').menuPopup('close', trigger_elm, false);
 						executeScript(data.params.hostId, data.params.scriptId, data.params.confirmation, trigger_elm);
 						cancelEvent(e);
 					};
@@ -907,6 +1099,37 @@ function getMenuPopupScriptData(scripts, hostId, trigger_elmnt) {
 	return getMenuPopupScriptItems(tree, trigger_elmnt);
 }
 
+/**
+ * Create menu for dashboard widget area selector.
+ *
+ * @param {object} area_selected  Area in which new widget will be created.
+ *
+ * @returns {array}
+ */
+function getDashboardWidgetActionMenu(area_selected) {
+	return [{
+		items: [{
+			label: t('Add widget'),
+			clickCallback: function() {
+				jQuery('.dashbrd-grid-container').dashboardGrid('addNewWidget', null, area_selected);
+			}
+		}, {
+			label: t('Paste widget'),
+			clickCallback: function() {
+				var widget_dims = {
+					x: area_selected.x,
+					y: area_selected.y,
+					width: area_selected.width,
+					height: area_selected.height
+				};
+
+				jQuery('.dashbrd-grid-container').dashboardGrid('pasteWidget', null, widget_dims);
+				jQuery('.dashbrd-grid-new-widget-placeholder').hide();
+			}
+		}]
+	}];
+}
+
 jQuery(function($) {
 
 	/**
@@ -916,6 +1139,9 @@ jQuery(function($) {
 	 * @param string sections[n]['label']  Section title (optional).
 	 * @param array  sections[n]['items']  Section menu data (see createMenuItem() for available options).
 	 * @param object event                 Menu popup call event.
+	 * @param object options               Menu popup options (optional).
+	 * @param object options['class']      Menu popup additional class name (optional).
+	 * @param object options['position']   Menu popup position object (optional).
 	 *
 	 * @see createMenuItem()
 	 */
@@ -928,156 +1154,163 @@ jQuery(function($) {
 		}
 	};
 
-	var methods = {
-		init: function(sections, event) {
-			var opener = $(this),
-				id = opener.data('menu-popup-id'),
-				menuPopup = $('#' + id),
-				mapContainer = null,
-				target = event.target;
-
-			if (event.type === 'contextmenu' || (IE && opener.closest('svg').length > 0)
-					|| event.originalEvent.detail !== 0) {
-				target = event;
+	/**
+	 * Created popup menu item nodes and append to $menu_popup.
+	 *
+	 * @param {object} $menu_popup    jQuery node of popup menu.
+	 * @param {array} sections        Array of menu popup sections.
+	 */
+	function addMenuPopupItems($menu_popup, sections) {
+		// Create menu sections.
+		$.each(sections, function(i, section) {
+			// Add a separator between menu item sections.
+			if (i > 0) {
+				$menu_popup.append($('<li>').append($('<div>')));
 			}
 
-			opener.attr('data-expanded', 'true');
+			var section_label = null;
 
-			// Close other action menus.
-			$('.action-menu-top').not('#' + id).menuPopup('close');
+			if (typeof section.label === 'string' && section.label.length) {
+				section_label = section.label;
+			}
 
-			if (menuPopup.length > 0) {
-				var display = menuPopup.css('display');
+			// Add menu item section label, if provided.
+			if (section_label !== null) {
+				$menu_popup.append($('<li>').append($('<h3>').text(section_label)));
+			}
 
-				// Hide current action menu sub-levels.
-				$('.action-menu', menuPopup).css('display', 'none');
-
-				if (display === 'block') {
-					menuPopup.fadeOut(0);
-					$(opener).removeAttr('data-expanded');
-				}
-				else {
-					menuPopup.fadeIn(50);
+			// Add individual menu items of the section.
+			$.each(section.items, function(i, item) {
+				item = $.extend({}, item);
+				if (sections.length > 1 && section_label !== null) {
+					item.ariaLabel = section_label + ', ' + item['label'];
 				}
+				$menu_popup.append(createMenuItem(item));
+			});
+		});
 
-				menuPopup.position({
-					of: target,
+		if (sections.length == 1) {
+			if (typeof sections[0].label === 'string' && sections[0].label.length) {
+				$menu_popup.attr({'aria-label': sections[0].label});
+			}
+		}
+	}
+
+	var defaultOptions = {
+		closeCallback: function(){}
+	};
+
+	var methods = {
+		init: function(sections, event, options) {
+			// Don't display empty menu.
+			if (!sections.length || !sections[0]['items'].length) {
+				return;
+			}
+
+			var $opener = $(this);
+
+			options = $.extend({
+				position: {
+					/*
+					 * Please note that click event is also triggered by hitting spacebar on the keyboard,
+					 * in which case the number of mouse clicks (stored in event.originalEvent.detail) will be zero.
+					 */
+					of: (event.type === 'click' && event.originalEvent.detail) ? event : event.target,
 					my: 'left top',
 					at: 'left bottom'
-				});
-			}
-			else {
-				id = new Date().getTime();
+				}
+			}, defaultOptions, options || {});
 
-				menuPopup = $('<ul>', {
-					'id': id,
+			// Close other action menus and prevent focus jumping before opening a new popup.
+			$('.menu-popup-top').menuPopup('close', null, false);
+
+			$opener.attr('aria-expanded', 'true');
+
+			var $menu_popup = $('<ul>', {
 					'role': 'menu',
-					'class': 'action-menu action-menu-top',
+					'class': 'menu-popup menu-popup-top',
 					'tabindex': 0
 				});
 
-				// create sections
-				var sections_length = sections.length;
-				if (sections_length) {
-					$.each(sections, function(i, section) {
-						if ((typeof section.label !== 'undefined') && (section.label.length > 0)) {
-							var h3 = $('<h3>').text(section.label);
-							var sectionItem = $('<li>').append(h3);
-						}
-
-						// Add section delimited for all sections except first one.
-						if (i > 0) {
-							menuPopup.append($('<li>').append($('<div>')));
-						}
-						menuPopup.append(sectionItem);
-
-						$.each(section.items, function(i, item) {
-							if (sections_length > 1) {
-								item['ariaLabel'] = section.label + ', ' + item['label'];
-							}
-							menuPopup.append(createMenuItem(item));
-						});
-					});
-				}
-
-				if (sections_length == 1) {
-					menuPopup.attr({'aria-label': sections[0].label});
-				}
-
-				// Skip displaying empty menu sections.
-				if (menuPopup.children().length == 0) {
-					return;
-				}
-
-				// Set menu popup for map area.
-				if (opener.prop('tagName') === 'AREA') {
-					$('.menuPopupContainer').remove();
-
-					mapContainer = $('<div>', {
-						'class': 'menuPopupContainer',
-						'css': {
-							position: 'absolute',
-							top: event.pageY,
-							left: event.pageX
-						}
-					})
-					.append(menuPopup);
-
-					$('body').append(mapContainer);
-				}
-				// Set menu popup for common html elements.
-				else {
-					opener.data('menu-popup-id', id);
-
-					$('body').append(menuPopup);
-				}
-
-				// Hide current action menu sub-levels.
-				$('.action-menu', menuPopup).css('display', 'none');
-
-				// display
-				menuPopup
-					.fadeIn(50)
-					.data('is-active', false)
-					.mouseenter(function() {
-						menuPopup.data('is-active', true);
-
-						clearTimeout(window.menuPopupTimeoutHandler);
-					})
-					.on('click', function(e) {
-						e.stopPropagation();
-					})
-					.position({
-						of: (opener.prop('tagName') === 'AREA') ? mapContainer : target,
-						my: 'left top',
-						at: 'left bottom'
-					});
+			// Add custom class, if specified.
+			if ('class' in options) {
+				$menu_popup.addClass(options.class);
 			}
 
-			addToOverlaysStack('contextmenu', event.target, 'contextmenu');
+			$opener.data({
+				sections: sections,
+				menu_popup: $menu_popup
+			});
+			addMenuPopupItems($menu_popup, sections);
 
-			$(document)
-				.on('click', {menu: menuPopup, opener: opener}, menuPopupDocumentCloseHandler)
-				.on('keydown', {menu: menuPopup}, menuPopupKeyDownHandler);
+			$menu_popup.data('menu_popup', options);
 
-			menuPopup.focus();
+			$('.wrapper').append($menu_popup);
+
+			// Position the menu (before hiding).
+			$menu_popup.position(options.position);
+
+			// Hide all action menu sub-levels, including the topmost, for fade effect to work.
+			$menu_popup.add('.menu-popup', $menu_popup).hide();
+
+			// Position and display the menu.
+			$menu_popup.fadeIn(50);
+
+			addToOverlaysStack('menu-popup', event.target, 'menu-popup');
+
+			// Need to be postponed.
+			setTimeout(function() {
+				$(document)
+					.on('click', {menu: $menu_popup, opener: $opener}, menuPopupDocumentCloseHandler)
+					.on('keydown', {menu: $menu_popup}, menuPopupKeyDownHandler);
+			});
+
+			$menu_popup.focus();
 		},
-		close: function(trigger_elmnt, return_focus) {
-			var menuPopup = $(this);
-			if (!menuPopup.is(trigger_elmnt) && menuPopup.has(trigger_elmnt).length === 0) {
-				menuPopup.data('is-active', false);
-				$(trigger_elmnt).removeAttr('data-expanded');
-				menuPopup.fadeOut(0);
 
-				$('.highlighted', menuPopup).removeClass('highlighted');
-				$('[aria-expanded="true"]', menuPopup).attr({'aria-expanded': 'false'});
+		close: function(trigger_elem, return_focus) {
+			var menu_popup = $(this),
+				options = $(menu_popup).data('menu_popup') || {};
+
+			if (!menu_popup.is(trigger_elem) && menu_popup.has(trigger_elem).length === 0) {
+				$('[aria-expanded="true"]', trigger_elem).attr({'aria-expanded': 'false'});
+				menu_popup.fadeOut(0);
+
+				$('.highlighted', menu_popup).removeClass('highlighted');
+				$('[aria-expanded="true"]', menu_popup).attr({'aria-expanded': 'false'});
 
 				$(document)
 					.off('click', menuPopupDocumentCloseHandler)
 					.off('keydown', menuPopupKeyDownHandler);
 
-				removeFromOverlaysStack('contextmenu', return_focus);
+				var overlay = removeFromOverlaysStack('menu-popup', return_focus);
+
+				if (overlay && typeof overlay['element'] !== undefined) {
+					// Remove expanded attribute of the original opener.
+					$(overlay['element']).attr({'aria-expanded': 'false'});
+				}
+
+				menu_popup.remove();
+
+				// Call menu close callback function.
+				typeof options.closeCallback === 'function' && options.closeCallback.apply();
 			}
+		},
+
+		/**
+		 * Refresh popup menu, call refreshCallback for every item if defined. Refresh recreate item dom nodes.
+		 */
+		refresh: function(widget) {
+			var $opener = $(this),
+				sections = $opener.data('sections'),
+				$menu_popup = $opener.data('menu_popup');
+
+			$menu_popup.empty();
+			sections.forEach(
+				section => section.items && section.items.forEach(
+					item => item.refreshCallback && item.refreshCallback.call(item, widget)
+			));
+			addMenuPopupItems($menu_popup, sections);
 		}
 	};
 
@@ -1087,7 +1320,7 @@ jQuery(function($) {
 	$.fn.actionMenuItemExpand = function() {
 		var li = $(this),
 			pos = li.position(),
-			menu = li.closest('.action-menu');
+			menu = li.closest('.menu-popup');
 
 		for (var item = $('li:first-child', menu); item.length > 0; item = item.next()) {
 			if (item[0] == li[0]) {
@@ -1127,13 +1360,13 @@ jQuery(function($) {
 	 */
 	$.fn.actionMenuItemCollapse = function() {
 		// Remove style and close sub-menus in deeper levels.
-		var parent_menu = $(this).closest('.action-menu');
+		var parent_menu = $(this).closest('.menu-popup');
 		$('.highlighted', parent_menu).removeClass('highlighted');
 		$('[aria-expanded]', parent_menu).attr({'aria-expanded': 'false'});
-		$('.action-menu', parent_menu).css({'display': 'none'});
+		$('.menu-popup', parent_menu).css({'display': 'none'});
 
 		// Close actual menu level.
-		parent_menu.not('.action-menu-top').css({'display': 'none'});
+		parent_menu.not('.menu-popup-top').css({'display': 'none'});
 		parent_menu.prev('[role="menuitem"]').attr({'aria-expanded': 'false'});
 
 		return this;
@@ -1144,7 +1377,7 @@ jQuery(function($) {
 	}
 
 	function menuPopupKeyDownHandler(event) {
-		var link_selector = '.action-menu-item',
+		var link_selector = '.menu-popup-item',
 			menu_popup = $(event.data.menu[0]),
 			level = menu_popup,
 			selected,
@@ -1157,14 +1390,14 @@ jQuery(function($) {
 
 		// Find active menu items.
 		items = $('>li', level).filter(function() {
-			return $(this).has('.action-menu-item').length;
+			return $(this).has('.menu-popup-item').length;
 		});
 
 		// Find an element that was selected when key was pressed.
-		if ($('.action-menu-item.highlighted', level).length) {
+		if ($('.menu-popup-item.highlighted', level).length) {
 			selected = $(link_selector + '.highlighted', level).closest('li');
 		}
-		else if ($('.action-menu-item', level).filter(function() {
+		else if ($('.menu-popup-item', level).filter(function() {
 			return this == document.activeElement;
 		}).length) {
 			selected = $(document.activeElement).closest('li');
@@ -1173,12 +1406,12 @@ jQuery(function($) {
 		// Perform action based on keydown event.
 		switch (event.which) {
 			case 37: // arrow left
-				if (typeof selected !== 'undefined' && selected.has('.action-menu')) {
+				if (typeof selected !== 'undefined' && selected.has('.menu-popup')) {
 					if (level != menu_popup) {
 						selected.actionMenuItemCollapse();
 
 						// Must focus previous element, otherwise screen reader will exit menu.
-						selected.closest('.action-menu').prev('[role="menuitem"]').addClass('highlighted').focus();
+						selected.closest('.menu-popup').prev('[role="menuitem"]').addClass('highlighted').focus();
 					}
 				}
 				break;
@@ -1202,7 +1435,7 @@ jQuery(function($) {
 				break;
 
 			case 39: // arrow right
-				if (typeof selected !== 'undefined' && selected.has('.action-menu')) {
+				if (typeof selected !== 'undefined' && selected.has('.menu-popup')) {
 					selected.actionMenuItemExpand();
 					$('ul > li ' + link_selector + ':first', selected).addClass('highlighted').focus();
 				}
@@ -1253,6 +1486,7 @@ jQuery(function($) {
 	 * @param string options['css']            Item class.
 	 * @param array  options['data']           Item data ("key" => "value").
 	 * @param array  options['items']          Item sub menu.
+	 * @param {bool} options['disabled']       Item disable status.
 	 * @param object options['clickCallback']  Item click callback.
 	 *
 	 * @return object
@@ -1261,7 +1495,8 @@ jQuery(function($) {
 		options = $.extend({
 			ariaLabel: options.label,
 			selected: false,
-			disabled: false
+			disabled: false,
+			class: false
 		}, options);
 
 		var item = $('<li>'),
@@ -1287,13 +1522,17 @@ jQuery(function($) {
 		}
 
 		if (options.disabled) {
-			link.addClass('action-menu-item-disabled');
+			link.addClass('menu-popup-item-disabled');
 		}
 		else {
-			link.addClass('action-menu-item');
+			link.addClass('menu-popup-item');
 
 			if (typeof options.url !== 'undefined') {
 				link.attr('href', options.url);
+
+				if ('target' in options) {
+					link.attr('target', options.target);
+				}
 			}
 
 			if (typeof options.clickCallback !== 'undefined') {
@@ -1305,19 +1544,26 @@ jQuery(function($) {
 			link.addClass('selected');
 		}
 
+		if (options.class) {
+			link.addClass(options.class);
+		}
+
 		if (typeof options.items !== 'undefined' && options.items.length > 0) {
 			link.attr({
 				'aria-haspopup': 'true',
 				'aria-expanded': 'false',
 				'area-hidden': 'true'
 			});
+			link.on('click', function(e) {
+				e.stopPropagation();
+			});
 		}
 
 		item.append(link);
 
 		if (typeof options.items !== 'undefined' && options.items.length > 0) {
 			var menu = $('<ul>', {
-					class : 'action-menu',
+					class : 'menu-popup',
 					role: 'menu'
 				})
 				.on('mouseenter', function(e) {
