@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,26 +20,15 @@
 
 
 require_once dirname(__FILE__).'/include/config.inc.php';
-require_once dirname(__FILE__).'/include/hostgroups.inc.php';
 require_once dirname(__FILE__).'/include/hosts.inc.php';
 require_once dirname(__FILE__).'/include/screens.inc.php';
 require_once dirname(__FILE__).'/include/forms.inc.php';
 require_once dirname(__FILE__).'/include/ident.inc.php';
 
-if (hasRequest('templates') && hasRequest('action') && getRequest('action') === 'template.export') {
-	$exportData = true;
-
-	$page['type'] = detect_page_type(PAGE_TYPE_XML);
-	$page['file'] = 'zbx_export_templates.xml';
-}
-else {
-	$exportData = false;
-
-	$page['type'] = detect_page_type(PAGE_TYPE_HTML);
-	$page['title'] = _('Configuration of templates');
-	$page['file'] = 'templates.php';
-	$page['scripts'] = ['multiselect.js'];
-}
+$page['type'] = detect_page_type(PAGE_TYPE_HTML);
+$page['title'] = _('Configuration of templates');
+$page['file'] = 'templates.php';
+$page['scripts'] = ['multiselect.js', 'textareaflexible.js', 'inputsecret.js', 'macrovalue.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -53,19 +42,26 @@ $fields = [
 	'templates'			=> [T_ZBX_INT, O_OPT, null,		DB_ID,	null],
 	'linked_templates'	=> [T_ZBX_INT, O_OPT, null,		DB_ID,	null],
 	'add_templates'		=> [T_ZBX_INT, O_OPT, null,		DB_ID,	null],
-	'add_template' 		=> [T_ZBX_STR, O_OPT, null,		null,	null],
 	'templateid'		=> [T_ZBX_INT, O_OPT, P_SYS,		DB_ID,	'isset({form}) && {form} == "update"'],
 	'template_name'		=> [T_ZBX_STR, O_OPT, null,		NOT_EMPTY, 'isset({add}) || isset({update})', _('Template name')],
 	'visiblename'		=> [T_ZBX_STR, O_OPT, null,		null,	'isset({add}) || isset({update})'],
-	'groupid'			=> [T_ZBX_INT, O_OPT, P_SYS,		DB_ID,	null],
+	'groupids'			=> [T_ZBX_INT, O_OPT, null,		DB_ID,	null],
 	'tags'				=> [T_ZBX_STR, O_OPT, null,		null,	null],
 	'mass_update_tags'	=> [T_ZBX_INT, O_OPT, null,		IN([ZBX_ACTION_ADD, ZBX_ACTION_REPLACE, ZBX_ACTION_REMOVE]),
 								null
 							],
 	'description'		=> [T_ZBX_STR, O_OPT, null,		null,	null],
 	'macros'			=> [T_ZBX_STR, O_OPT, P_SYS,		null,	null],
+	'mass_update_macros' => [T_ZBX_INT, O_OPT, null,
+								IN([ZBX_ACTION_ADD, ZBX_ACTION_REPLACE, ZBX_ACTION_REMOVE, ZBX_ACTION_REMOVE_ALL]),
+								null
+							],
+	'macros_add'		=> [T_ZBX_INT, O_OPT, null, IN([0,1]), null],
+	'macros_update'		=> [T_ZBX_INT, O_OPT, null, IN([0,1]), null],
+	'macros_remove'		=> [T_ZBX_INT, O_OPT, null, IN([0,1]), null],
+	'macros_remove_all' => [T_ZBX_INT, O_OPT, null, IN([0,1]), null],
 	'visible'			=> [T_ZBX_STR, O_OPT, null,			null,	null],
-	'mass_replace_tpls'	=> [T_ZBX_STR, O_OPT, null,			null,	null],
+	'mass_action_tpls'	=> [T_ZBX_INT, O_OPT, null, IN([ZBX_ACTION_ADD, ZBX_ACTION_REPLACE, ZBX_ACTION_REMOVE]), null ],
 	'mass_clear_tpls'	=> [T_ZBX_STR, O_OPT, null,			null,	null],
 	'show_inherited_macros' => [T_ZBX_INT, O_OPT, null,	IN([0,1]), null],
 	// actions
@@ -92,6 +88,7 @@ $fields = [
 	'filter_rst'		=> [T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'filter_name'		=> [T_ZBX_STR, O_OPT, null,		null,		null],
 	'filter_templates' =>  [T_ZBX_INT, O_OPT, null,		DB_ID,		null],
+	'filter_groups'		=> [T_ZBX_INT, O_OPT, null,		DB_ID,		null],
 	'filter_evaltype'	=> [T_ZBX_INT, O_OPT, null,
 								IN([TAG_EVAL_TYPE_AND_OR, TAG_EVAL_TYPE_OR]),
 								null
@@ -106,9 +103,6 @@ check_fields($fields);
 /*
  * Permissions
  */
-if (getRequest('groupid') && !isWritableHostGroups([getRequest('groupid')])) {
-	access_deny();
-}
 if (getRequest('templateid')) {
 	$templates = API::Template()->get([
 		'output' => [],
@@ -121,24 +115,6 @@ if (getRequest('templateid')) {
 	}
 }
 
-$templateIds = getRequest('templates', []);
-
-if ($exportData) {
-	$export = new CConfigurationExport(['templates' => $templateIds]);
-	$export->setBuilder(new CConfigurationExportBuilder());
-	$export->setWriter(CExportWriterFactory::getWriter(CExportWriterFactory::XML));
-	$exportData = $export->export();
-
-	if (hasErrorMesssages()) {
-		show_messages();
-	}
-	else {
-		print($exportData);
-	}
-
-	exit;
-}
-
 $tags = getRequest('tags', []);
 foreach ($tags as $key => $tag) {
 	// remove empty new tag lines
@@ -156,24 +132,19 @@ foreach ($tags as $key => $tag) {
 	}
 }
 
-// remove inherited macros data (actions: 'add', 'update' and 'form')
-if (hasRequest('macros')) {
-	$_REQUEST['macros'] = cleanInheritedMacros($_REQUEST['macros']);
+// Remove inherited macros data (actions: 'add', 'update' and 'form').
+$macros = cleanInheritedMacros(getRequest('macros', []));
 
-	// remove empty new macro lines
-	foreach ($_REQUEST['macros'] as $idx => $macro) {
-		if (!array_key_exists('hostmacroid', $macro) && $macro['macro'] === '' && $macro['value'] === '') {
-			unset($_REQUEST['macros'][$idx]);
-		}
-	}
-}
+// Remove empty new macro lines.
+$macros = array_filter($macros, function($macro) {
+	$keys = array_flip(['hostmacroid', 'macro', 'value', 'description']);
+
+	return (bool) array_filter(array_intersect_key($macro, $keys));
+});
 
 /*
  * Actions
  */
-if (hasRequest('add_template') && hasRequest('add_templates')) {
-	$_REQUEST['templates'] = array_merge($templateIds, $_REQUEST['add_templates']);
-}
 if (hasRequest('unlink') || hasRequest('unlink_and_clear')) {
 	$unlinkTemplates = [];
 
@@ -219,6 +190,23 @@ elseif (hasRequest('templateid') && (hasRequest('clone') || hasRequest('full_clo
 		$_REQUEST['groups'] = $groups;
 	}
 
+	if ($macros && in_array(ZBX_MACRO_TYPE_SECRET, array_column($macros, 'type'))) {
+		// Reset macro type and value.
+		$macros = array_map(function($value) {
+			return ($value['type'] == ZBX_MACRO_TYPE_SECRET)
+				? ['value' => '', 'type' => ZBX_MACRO_TYPE_TEXT] + $value
+				: $value;
+		}, $macros);
+
+		$msg = [
+			'type' => 'error',
+			'message' => _('The cloned template contains user defined macros with type "Secret text". The value and type of these macros were reset.'),
+			'src' => ''
+		];
+
+		echo makeMessageBox(false, [$msg], null, true, false)->addClass(ZBX_STYLE_MSG_WARNING);
+	}
+
 	if (hasRequest('clone')) {
 		unset($_REQUEST['templateid']);
 	}
@@ -239,7 +227,8 @@ elseif (hasRequest('action') && getRequest('action') === 'template.massupdate' &
 			$options['selectGroups'] = ['groupid'];
 		}
 
-		if (array_key_exists('linked_templates', $visible) && !hasRequest('mass_replace_tpls')) {
+		if (array_key_exists('linked_templates', $visible)
+				&& !(getRequest('mass_action_tpls') == ZBX_ACTION_REPLACE && !hasRequest('mass_clear_tpls'))) {
 			$options['selectParentTemplates'] = ['templateid'];
 		}
 
@@ -259,6 +248,15 @@ elseif (hasRequest('action') && getRequest('action') === 'template.massupdate' &
 			$tags = array_values($unique_tags);
 		}
 
+		if (array_key_exists('macros', $visible)) {
+			$mass_update_macros = getRequest('mass_update_macros', ZBX_ACTION_ADD);
+
+			if ($mass_update_macros == ZBX_ACTION_ADD || $mass_update_macros == ZBX_ACTION_REPLACE
+					|| $mass_update_macros == ZBX_ACTION_REMOVE) {
+				$options['selectMacros'] = ['hostmacroid', 'macro'];
+			}
+		}
+
 		$templates = API::Template()->get($options);
 
 		if (array_key_exists('groups', $visible)) {
@@ -302,27 +300,9 @@ elseif (hasRequest('action') && getRequest('action') === 'template.massupdate' &
 			$new_values['description'] = getRequest('description');
 		}
 
-		$linked_templateids = [];
-		if (array_key_exists('linked_templates', $visible)) {
-			$linked_templateids = getRequest('linked_templates', []);
-		}
-
-		if (hasRequest('mass_replace_tpls')) {
-			if (hasRequest('mass_clear_tpls')) {
-				$template_templates = API::Template()->get([
-					'output' => ['templateid'],
-					'hostids' => $templateids
-				]);
-
-				$template_templateids = zbx_objectValues($template_templates, 'templateid');
-				$templates_to_delete = array_diff($template_templateids, $linked_templateids);
-
-				$new_values['templates_clear'] = zbx_toObject($templates_to_delete, 'templateid');
-			}
-
-			$new_values['templates'] = $linked_templateids;
-		}
-
+		$template_macros_add = [];
+		$template_macros_update = [];
+		$template_macros_remove = [];
 		foreach ($templates as &$template) {
 			if (array_key_exists('groups', $visible)) {
 				if ($new_groupids && $mass_update_groups == ZBX_ACTION_ADD) {
@@ -340,10 +320,36 @@ elseif (hasRequest('action') && getRequest('action') === 'template.massupdate' &
 				}
 			}
 
-			if ($linked_templateids && array_key_exists('parentTemplates', $template)) {
-				$template['templates'] = array_unique(
-					array_merge($linked_templateids, zbx_objectValues($template['parentTemplates'], 'templateid'))
-				);
+			if (array_key_exists('linked_templates', $visible)) {
+				$parent_templateids = array_key_exists('parentTemplates', $template)
+					? zbx_objectValues($template['parentTemplates'], 'templateid')
+					: [];
+
+				switch (getRequest('mass_action_tpls')) {
+					case ZBX_ACTION_ADD:
+						$template['templates'] = array_unique(
+							array_merge($parent_templateids, getRequest('linked_templates', []))
+						);
+						break;
+
+					case ZBX_ACTION_REPLACE:
+						$template['templates'] = getRequest('linked_templates', []);
+						if (getRequest('mass_clear_tpls')) {
+							$template['templates_clear'] = array_unique(
+								array_diff($parent_templateids, getRequest('linked_templates', []))
+							);
+						}
+						break;
+
+					case ZBX_ACTION_REMOVE:
+						$template['templates'] = array_unique(
+							array_diff($parent_templateids, getRequest('linked_templates', []))
+						);
+						if (getRequest('mass_clear_tpls')) {
+							$template['templates_clear'] = array_unique(getRequest('linked_templates', []));
+						}
+						break;
+				}
 			}
 
 			if (array_key_exists('tags', $visible)) {
@@ -376,6 +382,81 @@ elseif (hasRequest('action') && getRequest('action') === 'template.massupdate' &
 				}
 			}
 
+			if (array_key_exists('macros', $visible)) {
+				switch ($mass_update_macros) {
+					case ZBX_ACTION_ADD:
+						if ($macros) {
+							$update_existing = getRequest('macros_add', 0);
+
+							foreach ($macros as $macro) {
+								foreach ($template['macros'] as $template_macro) {
+									if ($macro['macro'] === $template_macro['macro']) {
+										if ($update_existing) {
+											$macro['hostmacroid'] = $template_macro['hostmacroid'];
+											$template_macros_update[] = $macro;
+										}
+
+										continue 2;
+									}
+								}
+
+								$macro['hostid'] = $template['templateid'];
+								$template_macros_add[] = $macro;
+							}
+						}
+						break;
+
+					case ZBX_ACTION_REPLACE: // In Macros its update.
+						if ($macros) {
+							$add_missing = getRequest('macros_update', 0);
+
+							foreach ($macros as $macro) {
+								foreach ($template['macros'] as $template_macro) {
+									if ($macro['macro'] === $template_macro['macro']) {
+										$macro['hostmacroid'] = $template_macro['hostmacroid'];
+										$template_macros_update[] = $macro;
+
+										continue 2;
+									}
+								}
+
+								if ($add_missing) {
+									$macro['hostid'] = $template['templateid'];
+									$template_macros_add[] = $macro;
+								}
+							}
+						}
+						break;
+
+					case ZBX_ACTION_REMOVE:
+						if ($macros) {
+							$except_selected = getRequest('macros_remove', 0);
+
+							$macro_names = array_column($macros, 'macro');
+
+							foreach ($template['macros'] as $template_macro) {
+								if ((!$except_selected && in_array($template_macro['macro'], $macro_names))
+										|| ($except_selected && !in_array($template_macro['macro'], $macro_names))) {
+									$template_macros_remove[] = $template_macro['hostmacroid'];
+								}
+							}
+						}
+						break;
+
+					case ZBX_ACTION_REMOVE_ALL:
+						if (!getRequest('macros_remove_all', 0)) {
+							throw new Exception();
+						}
+
+						$template['macros'] = [];
+						break;
+				}
+
+				if ($mass_update_macros != ZBX_ACTION_REMOVE_ALL) {
+					unset($template['macros']);
+				}
+			}
+
 			unset($template['parentTemplates']);
 
 			$template = $new_values + $template;
@@ -386,6 +467,28 @@ elseif (hasRequest('action') && getRequest('action') === 'template.massupdate' &
 			throw new Exception();
 		}
 
+		/**
+		 * Macros must be updated separately, since calling API::UserMacro->replaceMacros() inside
+		 * API::Template->update() results in loss of secret macro values.
+		 */
+		if ($template_macros_remove) {
+			if (!API::UserMacro()->delete($template_macros_remove)) {
+				throw new Exception();
+			}
+		}
+
+		if ($template_macros_add) {
+			if (!API::UserMacro()->create($template_macros_add)) {
+				throw new Exception();
+			}
+		}
+
+		if ($template_macros_update) {
+			if (!API::UserMacro()->update($template_macros_update)) {
+				throw new Exception();
+			}
+		}
+
 		DBend(true);
 
 		uncheckTableRows();
@@ -442,11 +545,11 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			$groups = array_merge($groups, $new_groupid['groupids']);
 		}
 
-		// linked templates
-		$linkedTemplates = getRequest('templates', []);
+		// Linked templates.
 		$templates = [];
-		foreach ($linkedTemplates as $linkedTemplateId) {
-			$templates[] = ['templateid' => $linkedTemplateId];
+
+		foreach (array_merge(getRequest('templates', []), getRequest('add_templates', [])) as $templateid) {
+			$templates[] = ['templateid' => $templateid];
 		}
 
 		$templatesClear = getRequest('clear_templates', []);
@@ -459,7 +562,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			'name' => getRequest('visiblename', ''),
 			'groups' => zbx_toObject($groups, 'groupid'),
 			'templates' => $templates,
-			'macros' => getRequest('macros', []),
+			'macros' => $macros,
 			'tags' => $tags,
 			'description' => getRequest('description', '')
 		];
@@ -642,32 +745,36 @@ elseif (hasRequest('templates') && hasRequest('action') && str_in_array(getReque
 	if ($result) {
 		uncheckTableRows();
 	}
+	else {
+		$templateids = API::Template()->get([
+			'output' => [],
+			'templateids' => $templates,
+			'editable' => true
+		]);
+		uncheckTableRows(null, zbx_objectValues($templateids, 'templateid'));
+	}
 	show_messages($result, _('Template deleted'), _('Cannot delete template'));
 }
 
 /*
  * Display
  */
-$pageFilter = new CPageFilter([
-	'config' => [
-		'individual' => 1
-	],
-	'groups' => [
-		'templated_hosts' => true,
-		'editable' => true
-	],
-	'groupid' => getRequest('groupid')
-]);
-$_REQUEST['groupid'] = $pageFilter->groupid;
-
 if (hasRequest('templates') && (getRequest('action') === 'template.massupdateform' || hasRequest('masssave'))) {
 	$data = [
 		'templates' => getRequest('templates', []),
 		'visible' => getRequest('visible', []),
-		'mass_replace_tpls' => getRequest('mass_replace_tpls'),
+		'mass_action_tpls' => getRequest('mass_action_tpls'),
 		'mass_clear_tpls' => getRequest('mass_clear_tpls'),
 		'groups' => getRequest('groups', []),
 		'tags' => $tags,
+		'macros' => $macros,
+		'macros_checkbox' => [
+			ZBX_ACTION_ADD => getRequest('macros_add', '0'),
+			ZBX_ACTION_REPLACE => getRequest('macros_update', '0'),
+			ZBX_ACTION_REMOVE => getRequest('macros_remove', '0'),
+			ZBX_ACTION_REMOVE_ALL => getRequest('macros_remove_all', '0')
+		],
+		'macros_visible' => getRequest('mass_update_macros', ZBX_ACTION_ADD),
 		'description' => getRequest('description'),
 		'linked_templates' => getRequest('linked_templates', [])
 	];
@@ -687,17 +794,31 @@ if (hasRequest('templates') && (getRequest('action') === 'template.massupdatefor
 		]), ['templateid' => 'id'])
 		: [];
 
+	$data['groups'] = $data['groups']
+		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
+			'groupids' => $data['groups'],
+			'editable' => true
+		]), ['groupid' => 'id'])
+		: [];
+
+	if (!$data['macros']) {
+		$data['macros'] = [['macro' => '', 'type' => ZBX_MACRO_TYPE_TEXT, 'value' => '', 'description' => '']];
+	}
+
 	$view = new CView('configuration.template.massupdate', $data);
 }
 elseif (hasRequest('form')) {
 	$data = [
 		'form' => getRequest('form'),
 		'templateid' => getRequest('templateid', 0),
-		'linked_templates' => getRequest('templates', []),
+		'linked_templates' => [],
+		'add_templates' => [],
 		'original_templates' => [],
-		'parent_templates' => [],
 		'tags' => $tags,
-		'show_inherited_macros' => getRequest('show_inherited_macros', 0)
+		'show_inherited_macros' => getRequest('show_inherited_macros', 0),
+		'readonly' => false,
+		'macros' => $macros
 	];
 
 	if ($data['templateid'] != 0) {
@@ -711,13 +832,13 @@ elseif (hasRequest('form')) {
 		]);
 		$data['dbTemplate'] = reset($dbTemplates);
 
-		$data['original_templates'] = [];
 		foreach ($data['dbTemplate']['parentTemplates'] as $parentTemplate) {
 			$data['original_templates'][$parentTemplate['templateid']] = $parentTemplate['templateid'];
 		}
 
 		if (!hasRequest('form_refresh')) {
 			$data['tags'] = $data['dbTemplate']['tags'];
+			$data['macros'] = $data['dbTemplate']['macros'];
 		}
 	}
 
@@ -734,32 +855,61 @@ elseif (hasRequest('form')) {
 		CArrayHelper::sort($data['tags'], ['tag', 'value']);
 	}
 
-	$templateIds = getRequest('templates', hasRequest('form_refresh') ? [] : $data['original_templates']);
+	// Add already linked and new templates.
+	$templates = [];
+	$request_linked_templates = getRequest('templates', hasRequest('form_refresh') ? [] : $data['original_templates']);
+	$request_add_templates = getRequest('add_templates', []);
 
-	// Get linked templates.
-	$data['linkedTemplates'] = API::Template()->get([
-		'output' => ['templateid', 'name'],
-		'templateids' => $templateIds,
-		'preservekeys' => true
-	]);
+	if ($request_linked_templates || $request_add_templates) {
+		$templates = API::Template()->get([
+			'output' => ['templateid', 'name'],
+			'templateids' => array_merge($request_linked_templates, $request_add_templates),
+			'preservekeys' => true
+		]);
+
+		$data['linked_templates'] = array_intersect_key($templates, array_flip($request_linked_templates));
+		CArrayHelper::sort($data['linked_templates'], ['name']);
+
+		$data['add_templates'] = array_intersect_key($templates, array_flip($request_add_templates));
+
+		foreach ($data['add_templates'] as &$template) {
+			$template = CArrayHelper::renameKeys($template, ['templateid' => 'id']);
+		}
+		unset($template);
+	}
 
 	$data['writable_templates'] = API::Template()->get([
 		'output' => ['templateid'],
-		'templateids' => $templateIds,
+		'templateids' => array_keys($data['linked_templates']),
 		'editable' => true,
 		'preservekeys' => true
 	]);
 
-	CArrayHelper::sort($data['linkedTemplates'], ['name']);
+	// Add inherited macros to template macros.
+	if ($data['show_inherited_macros']) {
+		$data['macros'] = mergeInheritedMacros($data['macros'], getInheritedMacros(array_keys($templates)));
+	}
 
-	$groups = [];
+	// Sort only after inherited macros are added. Otherwise the list will look chaotic.
+	$data['macros'] = array_values(order_macros($data['macros'], 'macro'));
+
+	// The empty inputs will not be shown if there are inherited macros, for example.
+	if (!$data['macros']) {
+		$macro = ['macro' => '', 'value' => '', 'description' => '', 'type' => ZBX_MACRO_TYPE_TEXT];
+
+		if ($data['show_inherited_macros']) {
+			$macro['inherited_type'] = ZBX_PROPERTY_OWN;
+		}
+
+		$data['macros'][] = $macro;
+	}
 
 	if (!hasRequest('form_refresh')) {
 		if ($data['templateid'] != 0) {
 			$groups = zbx_objectValues($data['dbTemplate']['groups'], 'groupid');
 		}
-		elseif (getRequest('groupid', 0) != 0) {
-			$groups[] = getRequest('groupid');
+		else {
+			$groups = getRequest('groupids', []);
 		}
 	}
 	else {
@@ -816,6 +966,12 @@ elseif (hasRequest('form')) {
 	}
 	CArrayHelper::sort($data['groups_ms'], ['name']);
 
+	// This data is used in common.template.edit.js.php.
+	$data['macros_tab'] = [
+		'linked_templates' => array_map('strval', array_keys($data['linked_templates'])),
+		'add_templates' => array_map('strval', array_keys($data['add_templates']))
+	];
+
 	$view = new CView('configuration.template.edit', $data);
 }
 else {
@@ -829,6 +985,7 @@ else {
 	if (hasRequest('filter_set')) {
 		CProfile::update('web.templates.filter_name', getRequest('filter_name', ''), PROFILE_TYPE_STR);
 		CProfile::updateArray('web.templates.filter_templates', getRequest('filter_templates', []), PROFILE_TYPE_ID);
+		CProfile::updateArray('web.templates.filter_groups', getRequest('filter_groups', []), PROFILE_TYPE_ID);
 		CProfile::update('web.templates.filter.evaltype', getRequest('filter_evaltype', TAG_EVAL_TYPE_AND_OR),
 			PROFILE_TYPE_INT
 		);
@@ -850,6 +1007,7 @@ else {
 	elseif (hasRequest('filter_rst')) {
 		CProfile::delete('web.templates.filter_name');
 		CProfile::deleteIdx('web.templates.filter_templates');
+		CProfile::deleteIdx('web.templates.filter_groups');
 		CProfile::delete('web.templates.filter.evaltype');
 		CProfile::deleteIdx('web.templates.filter.tags.tag');
 		CProfile::deleteIdx('web.templates.filter.tags.value');
@@ -859,6 +1017,7 @@ else {
 	$filter = [
 		'name' => CProfile::get('web.templates.filter_name', ''),
 		'templates' => CProfile::getArray('web.templates.filter_templates', null),
+		'groups' => CProfile::getArray('web.templates.filter_groups', null),
 		'evaltype' => CProfile::get('web.templates.filter.evaltype', TAG_EVAL_TYPE_AND_OR),
 		'tags' => []
 	];
@@ -873,9 +1032,6 @@ else {
 
 	$config = select_config();
 
-	// get templates
-	$templates = [];
-
 	$filter['templates'] = $filter['templates']
 		? CArrayHelper::renameObjectsKeys(API::Template()->get([
 			'output' => ['templateid', 'name'],
@@ -884,27 +1040,53 @@ else {
 		]), ['templateid' => 'id'])
 		: [];
 
-	if ($pageFilter->groupsSelected) {
-		$templates = API::Template()->get([
-			'output' => ['templateid', $sortField],
-			'evaltype' => $filter['evaltype'],
-			'tags' => $filter['tags'],
-			'search' => [
-				'name' => ($filter['name'] === '') ? null : $filter['name']
-			],
-			'parentTemplateids' => $filter['templates'] ? array_keys($filter['templates']) : null,
-			'groupids' => $pageFilter->groupids,
+	// Get host groups.
+	$filter['groups'] = $filter['groups']
+		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
+			'groupids' => $filter['groups'],
+			'templated_hosts' => true,
 			'editable' => true,
-			'sortfield' => $sortField,
-			'limit' => $config['search_limit'] + 1
-		]);
+			'preservekeys' => true
+		]), ['groupid' => 'id'])
+		: [];
+
+	$filter_groupids = $filter['groups'] ? array_keys($filter['groups']) : null;
+	if ($filter_groupids) {
+		$filter_groupids = getSubGroups($filter_groupids);
 	}
+
+	// Select templates.
+	$templates = API::Template()->get([
+		'output' => ['templateid', $sortField],
+		'evaltype' => $filter['evaltype'],
+		'tags' => $filter['tags'],
+		'search' => [
+			'name' => ($filter['name'] === '') ? null : $filter['name']
+		],
+		'parentTemplateids' => $filter['templates'] ? array_keys($filter['templates']) : null,
+		'groupids' => $filter_groupids,
+		'editable' => true,
+		'sortfield' => $sortField,
+		'limit' => $config['search_limit'] + 1
+	]);
+
 	order_result($templates, $sortField, $sortOrder);
 
-	$url = (new CUrl('templates.php'))
-		->setArgument('groupid', getRequest('groupid', 0));
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
+	}
+	else {
+		$page_num = CPagerHelper::loadPage($page['file']);
+	}
 
-	$paging = getPagingLine($templates, $sortOrder, $url);
+	CPagerHelper::savePage($page['file'], $page_num);
+
+	$paging = CPagerHelper::paginate($page_num, $templates, $sortOrder, new CUrl('templates.php'));
 
 	$templates = API::Template()->get([
 		'output' => ['templateid', 'name'],
@@ -964,9 +1146,9 @@ else {
 	}
 
 	$data = [
-		'pageFilter' => $pageFilter,
 		'templates' => $templates,
 		'paging' => $paging,
+		'page' => $page_num,
 		'filter' => $filter,
 		'sortField' => $sortField,
 		'sortOrder' => $sortOrder,
@@ -983,7 +1165,6 @@ else {
 	$view = new CView('configuration.template.list', $data);
 }
 
-$view->render();
-$view->show();
+echo $view->getOutput();
 
 require_once dirname(__FILE__).'/include/page_footer.php';
